package org.halfcat.halflife;

import java.text.*;
import java.io.*;
import java.net.*;
import java.util.*;
import org.halfcat.persistence.*;
/**
 * Singleton (for now) bound to a listener port -- ugly, but will work
 *
 * Creation date: (8/10/00 3:28:04 PM)
 * @author: Chandler Howell
 */
public class LogMessageListener
	implements Runnable
{
	private static LogMessageListener theListener;
	
	int listenPort = 4444;
	DatagramSocket theServer = null;

	// Hashtable, key "w.x.y.z:27015", value is a ParserThread
	Hashtable listenerTable = new Hashtable();

	// string to prepend to each file. i.e. "cstrike-"
	String filePrefix = "";

	// whether or not to echo the log file to the console
	boolean echoLogs = false;

	// if true, use the fileprefix + a sequence # to identify
	// connections when echoing logs (cuts the line width down a LOT)
	boolean shortName = true;

	// if true, check each line for "Log closed" message and
	// create a new file
	boolean checkClose = true;

	RconServer rconServer = null;

	String address = "192.168.1.10";
//	String lastAddress = "192.168.1.10";

	private Hashtable serverRecords = new Hashtable();

	private boolean initialized = false;

	private boolean debug;

	public static final int SAY_ALIVE      =  0;
	public static final int SAY_DEAD       =  1;
	public static final int SAY_ALIVE_CT   =  2;
	public static final int SAY_ALIVE_T    =  3;
	public static final int SAY_DEAD_CT    =  4;
	public static final int SAY_DEAD_T     =  5;


	public static final int MSG_ENTER      =  6;
	public static final int MSG_LEAVE      =  7;
	public static final int MSG_DEATH      =  8;
	public static final int MSG_BOMB_PLANT =  9;
	public static final int MSG_HOSTAGE    = 10;
	public static final int MSG_END_ROUND  = 11;

	public static final int SAY_CONSOLE    = 12;
	
	
	public static String[] textColors = null;

	boolean portRemappingEnabled = false;

	boolean isListening = false;
/**
 * ListenServer constructor comment.
 */
private LogMessageListener()
{
	super();	
	initialize();
}
/**
 * ListenServer constructor comment.
 */
private LogMessageListener( int port )
{
	super();
	listenPort = port;
	initialize();
}
/**
 * Insert the method's description here.
 * Creation date: (8/26/00 7:37:49 AM)
 * @param w java.io.Writer
 */
public void addListener( String whichServer, OutputStream  os ) 
{
	addListener( whichServer, new OutputStreamWriter( os ) );
}
/**
 * Insert the method's description here.
 * Creation date: (8/26/00 7:37:49 AM)
 * @param w java.io.Writer
 */
public void addListener( String whichServer, Writer w ) 
{
	// nasty, but beats changing the API at this point

	int index = whichServer.indexOf( ':' );
	String address = whichServer.substring( 0, index );
	int port = Integer.parseInt( whichServer.substring( index + 1 ) );

	ServerRecord sr = getServerRecord( address, port );
	ParserThread pt = getParser( whichServer );
	
	pt.addListener( w );
	
	if ( debug )
		System.out.println( "Adding Listener for " + whichServer + ", now " + pt.getListenerCount() + " listeners" );
}
/**
 * Insert the method's description here.
 * Creation date: (8/10/00 3:32:59 PM)
 */
private void bindToPort( int portNumber )
	throws java.io.IOException
{
	System.out.println( "Binding to port: " + portNumber );
	listenPort = portNumber;
	theServer = new DatagramSocket( listenPort );
}
/**
 * Insert the method's description here.
 * Creation date: (8/26/00 8:16:52 AM)
 */
public void finalize()
	throws Throwable
{
	theServer.close();
	super.finalize();
}
/**
 * Insert the method's description here.
 * Creation date: (8/31/00 12:22:38 PM)
 * @return org.halfcat.halflife.LogMessageListener
 */
public synchronized static LogMessageListener getInstance() 
{
	if ( theListener == null )
		theListener = new LogMessageListener();

	return theListener;
}
/**
 * Insert the method's description here.
 * Creation date: (8/31/00 12:22:38 PM)
 * @return org.halfcat.halflife.LogMessageListener
 */
public synchronized static LogMessageListener getInstance( int listenPort ) 
{
	if ( theListener == null )
		theListener = new LogMessageListener( listenPort );
	else
		System.out.println( "WARNING:  logListener already listening on port " + listenPort );

	return theListener;
}
/**
 * Insert the method's description here.
 * Creation date: (8/28/00 10:10:11 AM)
 * @return org.halfcat.halflife.ParserThread
 * @param key java.lang.String
 */
private ParserThread getParser( String key ) 
{
	ParserThread pt;
	if ( ! listenerTable.containsKey( key ) )
	{
		if ( debug )
			System.out.println( "Creating new parser for key " + key );
		// create & start a new parser for the thread
		pt = new ParserThread();
		
		// the serverRecord has already been created at this point
		// just need to find it
		int index = key.indexOf( ':' );
		String address = key.substring( 0, index );
		int port = Integer.parseInt( key.substring( index + 1 ) );
		
		pt.setServerRecord( getServerRecord( address, port ) );
		listenerTable.put( key, pt );
	}
	else
		pt = (ParserThread)listenerTable.get( key );

	return pt;
}
/**
 * Insert the method's description here.
 * Creation date: (8/30/00 10:09:48 PM)
 * @return org.halfcat.halflife.ServerRecord
 * @param address java.lang.String
 * @param port int
 */
private ServerRecord getServerRecord(String address, int port)
{
	// this needs to happen before we create a new ParserThread
	// it's ugly, but it works
	String key = address + ":" + port;
	ServerRecord sr = null;
	if ( serverRecords.containsKey( key ) )
		sr = (ServerRecord)serverRecords.get( key );
	else
	{
		// ServerRecord not cached
		if ( debug )
			System.out.println( "No cached record for " + address + ":" + port );
		try
		{
			List l = DataManager.fetchLimited( ServerRecord.class, "address", address );
			Iterator i = l.iterator();
			while ( i.hasNext() )
			{
				sr = (ServerRecord)i.next();
				if ( sr.getPort() == port )
					break;
				else
					sr = null;
			}
		}
		catch( Exception e )
		{
			e.printStackTrace();
		}
		if ( sr == null )
		{
			if ( debug )
				System.out.println( "Creating new ServerRecord " + address + ":" + port );
			sr = new ServerRecord();
			sr.setAddress( address );
			sr.setPort( port );
			// store in DB
			try
			{
				// Weird and hacked workaround to limitations in my
				// RDAL
				DataManager.save( sr );
				sr.setObjectState( ObjectState.DIRTY );
				DataManager.save( sr );
			}
			catch( Exception ee )
			{
				ee.printStackTrace();
			}
		}				
		serverRecords.put( key, sr );
	} // endif serverRecords not cached
	return sr;
}
/**
 * Insert the method's description here.
 * Creation date: (8/26/00 7:35:14 AM)
 */
public synchronized void initialize() 
{
//RR GG; BB
	textColors = new String[13];
	
	textColors[SAY_ALIVE] = "#333333";
	textColors[SAY_DEAD] = "#666666";
	textColors[SAY_ALIVE_CT] = "#000099";
	textColors[SAY_ALIVE_T] = "#990000";

	textColors[SAY_DEAD_CT] = "#000077";
	textColors[SAY_DEAD_T] = "#770000";

	textColors[MSG_ENTER] = "#000000";
	textColors[MSG_LEAVE] = "#000000";
	textColors[MSG_DEATH] = "#003300";
	textColors[MSG_BOMB_PLANT] = "#22FF22";
	textColors[MSG_HOSTAGE] = "#22FF22";
	textColors[MSG_END_ROUND] = "#333333";

	textColors[SAY_CONSOLE] = "#000000";

	System.out.println( "LogMessageListener initializing" );
	
	try
	{
		if ( ! initialized )
		{
			bindToPort( listenPort );
			
			initialized = true;
			isListening = true;
						
			Thread t = new Thread( this );
			t.start();
		}
	}
	catch( Exception e )
	{
		System.out.println( "Initialization failed" );

		try
		{
			theServer.close();
		}
		catch( Exception ee ) { ; }
		
		e.printStackTrace();
		initialized = false;
		isListening = false;
	}
}
/**
 * Insert the method's description here.
 * Creation date: (9/4/00 8:15:33 AM)
 * @return boolean
 */
public boolean isDebug() {
	return debug;
}
/**
 * Returns true if the Listener is bound to a port
 * Creation date: (9/5/00 5:42:48 PM)
 * @return boolean
 */
public boolean isListening() 
{
	return isListening;
}
/**
 * Insert the method's description here.
 * Creation date: (9/3/00 11:12:07 PM)
 * @return boolean
 */
public boolean isPortRemappingEnabled() 
{
	return portRemappingEnabled;
}
/**
 * Insert the method's description here.
 * Creation date: (8/10/00 3:28:19 PM)
 * @param args java.lang.String[]
 */
public static void main(String[] args) 
{

	String dbURL = DataSource.createDbUrl( "192.168.1.7", "halfcat4", null );
	DataSource.connectToDB( dbURL, "howellc", "howellc" );

	
//	ls.setOutStream( System.out );
	try
	{
		LogMessageListener ls = new LogMessageListener();
		ls.setPortRemappingEnabled( true );
		ls.setDebug( true );
		ls.initialize();
		
		ls.addListener( "192.168.1.7:27017", System.out );
		ls.addListener( "63.89.102.206:27017", System.out );
	}
	catch( Exception ioe )
	{
		ioe.printStackTrace();
	}
}
/**
 * run method comment.
 */
public void run() 
{
	System.out.println( "Listener Thread Starting" );

	Hashtable portMap = new Hashtable();;
	if ( portRemappingEnabled )
	{
		System.out.println( "-Port Remapping enabled" );
	}

	boolean running = true;
	while ( running )
	{
		try
		{
			byte[] buf = new byte[256];

			// receive request
			DatagramPacket packet = new DatagramPacket( buf, buf.length );

			// this blocks until data is received
			theServer.receive( packet );
			InetAddress address = packet.getAddress();
			String addr = address.getHostAddress();

			int port = packet.getPort();
			String key = addr + ":" + port;

			if ( debug )
				System.out.println( "Raw logListener key: " + key );
				
			if ( portRemappingEnabled )
			{
				if ( portMap.containsKey( key ) )
				{
					port = ((Integer)portMap.get( key )).intValue();

					if ( debug )
						System.out.print( key + " already remapped to " );

					key = addr + ":" + port;

					if ( debug )	
						System.out.println( key );
				}
				else
				{
					if ( debug )
						System.out.println( "Checking server at " + addr + ":" + port );
					StatusServer ss = new StatusServer( addr, port );
					int qp;
					try
					{
 						ss.refresh();
						// query port is the port that the packets come from
						qp = ss.getQueryPort();
						

						// port is the port that the server thinks it's listening on
						port = ss.getPort();
						if ( debug )
							System.out.println( "Remapping " + qp + " to " + port );
						if ( port != 0 )
						{
							portMap.put( key, new Integer( port ) );
							key = addr + ":" + port;
						}
						else if ( debug )
							System.out.println( "port was 0...not remapped" );
						
					}
					catch( Exception e )
					{
						if ( debug )
						  System.out.println( "Failed to refresh server" );
					}					
				}
			}	// if ( portRemappingEnabled )
			else if ( debug )
				System.out.println( "Not Remapped" );

			// this needs to happen before we create a new ParserThread
			// it's ugly, but it works
			ServerRecord sr = getServerRecord( addr, port );

			if ( debug )
				System.out.println( "logListener key: " + key );

			// this grabs just the valid portion of the string, including the newline
			// from the logstream
			String line = new String( packet.getData(), 8, (packet.getLength() - 9) );
			 
			ParserThread pt = (ParserThread)getParser( key );
			if ( debug )
				System.out.println( "Listeners: " + pt.getListenerCount() );
				
			pt.parse( line );
		}
		catch (IOException e)
		{
			System.out.println( "LogMessageListener.run()" );
			if ( debug )
				e.printStackTrace();
		}
	}
}
/**
 * "Say" a message as the server console
 * Creation date: (8/29/00 3:41:28 PM)
 * @param key java.lang.String
 * @param message java.lang.String
 */
public void sayConsole(String key, String message) 
{
	getParser( key ).say( "<b>console:</b> " + message );
}
/**
 * Insert the method's description here.
 * Creation date: (9/4/00 8:15:33 AM)
 * @param newDebug boolean
 */
public void setDebug(boolean newDebug) 
{
	debug = newDebug;
	System.out.println( "Debugging now " + newDebug );
}
/**
 * Insert the method's description here.
 * Creation date: (9/3/00 11:12:07 PM)
 * @param newPortRemappingEnabled boolean
 */
public void setPortRemappingEnabled( boolean newPortRemappingEnabled ) 
{
	portRemappingEnabled = newPortRemappingEnabled;
	System.out.println( "port Remapping now " + newPortRemappingEnabled );
}
}
