package org.halfcat.halflife;

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;

import org.halfcat.halflife.cstrike.*;
import org.halfcat.persistence.*;
/**
 * Insert the type's description here.
 * Creation date: (8/10/00 3:37:14 PM)
 * @author: Chandler Howell
 */
public class ParserThread
	implements Runnable
{
	public static boolean debug = false;
	
	String idString = "";

	SimpleDateFormat sdf = new SimpleDateFormat( "MM/dd/yyyy - hh:mm:ss" );

	// the current game this thread is processing
	Game g;
	ServerRecord serverRecord;

	// List of writers 
	List listenerList = new ArrayList();
	
	public static final int SAY_ALIVE      =  0;
	public static final int SAY_DEAD       =  1;
	public static final int SAY_ALIVE_CT   =  2;
	public static final int SAY_ALIVE_T    =  3;
	public static final int SAY_DEAD_CT    =  4;
	public static final int SAY_DEAD_T     =  5;

	public static final int MSG_ENTER      =  6;
	public static final int MSG_LEAVE      =  7;
	public static final int MSG_DEATH      =  8;
	public static final int MSG_BOMB_PLANT =  9;
	public static final int MSG_HOSTAGE    = 10;
	public static final int MSG_END_ROUND  = 11;

	public static final int SAY_CONSOLE    = 12;
	
	
	public static String[] textColors = null;

	// for running as a log processor
	private static String dbHost = "192.168.1.7";
	private static String dbName = "halfcat";
	private static String dbLogin = "howellc";
	private static String dbPasswd = "howellc";
	private static int dbPort = 5432;

	private static String serverHost = "192.168.1.7";
	private static int serverPort = 27017;


	private static String loadDirectory = null;
	/** Flag for whether or not to process log text,
	 *  or simply to process people's say & say_team
	 *  and then return
	 */
	public static boolean processing = true;
	private static boolean daemonMode = false;
	static String processedDir = null;

	// used to get summaries when running in std-in mode
	private static boolean summary = false;

	private static int filesLoaded = 0;
	private static int filesSkipped = 0;
	private static int filesErrored = 0;

	File procDir = null;

	long sleepTimeInMillis = 10000;
/**
 * ListenerSocket constructor comment.
 */
public ParserThread() 
{
	super();

	g = new Game();

	//RR GG; BB
	textColors = new String[13];
	
	textColors[SAY_ALIVE] = "#333333";
	textColors[SAY_DEAD] = "#666666";
	textColors[SAY_ALIVE_CT] = "#000099";
	textColors[SAY_ALIVE_T] = "#990000";

	textColors[SAY_DEAD_CT] = "#000077";
	textColors[SAY_DEAD_T] = "#770000";

	textColors[MSG_ENTER] = "#000000";
	textColors[MSG_LEAVE] = "#000000";
	textColors[MSG_DEATH] = "#003300";
	textColors[MSG_BOMB_PLANT] = "#22FF22";
	textColors[MSG_HOSTAGE] = "#22FF22";
	textColors[MSG_END_ROUND] = "#333333";

	textColors[SAY_CONSOLE] = "#000000";
	
}
/**
 * Insert the method's description here.
 * Creation date: (8/26/00 7:37:49 AM)
 * @param w java.io.Writer
 */
public void addListener( OutputStream os ) 
{
	listenerList.add( new OutputStreamWriter( os ) );
}
/**
 * Insert the method's description here.
 * Creation date: (8/26/00 7:37:49 AM)
 * @param w java.io.Writer
 */
public void addListener( Writer w ) 
{
	listenerList.add( w );
}
/**
 * Insert the method's description here.
 * Creation date: (8/10/00 4:26:22 PM)
 * @return java.lang.String
 */
public java.lang.String getIdString() {
	return idString;
}
/**
 * Insert the method's description here.
 * Creation date: (9/4/00 1:00:01 PM)
 * @return int
 */
public int getListenerCount() 
{
	return listenerList.size();
}
/**
 * Insert the method's description here.
 * Creation date: (8/30/00 8:41:37 AM)
 * @return org.halfcat.halflife.ServerRecord
 */
public ServerRecord getServerRecord() 
{
	return serverRecord;
}
/**
 * Insert the method's description here.
 * Creation date: (9/3/00 12:44:18 PM)
 * @return long
 */
public long getSleepTimeInMillis() {
	return sleepTimeInMillis;
}
/**
 * If true, then then ParserThread is being used to
 * load the database
 * Creation date: (9/1/00 6:55:48 AM)
 * @return boolean
 */
public static boolean isProcessing() 
{
	return processing;
}
/**
 * Load all the files in a directory
 * Creation date: (9/3/00 12:37:59 PM)
 * @param dirName java.lang.String The full pathname of the directory
 */
private void loadDirectory( File fd ) 
{
	
	if ( fd.isDirectory()	)
	{
//		BufferedReader fr;
		File[] fa = fd.listFiles();
		System.out.println( "Processing " + fa.length + " files from " + fd.getAbsolutePath() );
		for ( int x = 0; x < fa.length; x++ )
		{						
			if ( ! fa[x].isDirectory() && fa[x].length() > 0 )
				processFile( fa[x], this, true );
			else
				filesSkipped++;
		}
	}
	else
	{
		System.out.println( "Not a directory: " + fd.getAbsolutePath() );
	}
	
}
/**
 * Insert the method's description here.
 * Creation date: (9/1/00 7:09:01 AM)
 * @param parserFile java.lang.String
 */
private void loadParserProperties( String parserFile ) 
{
	if ( parserFile == null )
		return;


	Properties p = new Properties();
		
	try
	{
		p.load( new FileInputStream( parserFile ) );
	}
	catch( FileNotFoundException fnfe )
	{
		System.out.println( "No such file: " + parserFile + " ...Exiting" );
		System.exit( -1 );
	}
	catch( IOException ioe )
	{
		System.out.println( "Unable to process " + parserFile + " ...Exiting" );
		System.exit( -1 );
	}
		
	dbLogin  = p.getProperty( "parser.dbLogin" );
	dbPasswd = p.getProperty( "parser.dbPassword" );
	dbHost   = p.getProperty( "parser.dbHost" );
	dbName   = p.getProperty( "parser.dbName" );
	
	try
	{
		dbPort = Integer.parseInt( p.getProperty( "parser.dbPort", "5432" ) );
	}
	catch( Exception e )
	{
		dbPort = 5432;
	}

	try
	{
		daemonMode = Boolean.valueOf( p.getProperty( "parser.daemon", "false" ) ).booleanValue();
	}
	catch( Exception e )
	{
		daemonMode = false;
	}

	try
	{
		sleepTimeInMillis = Long.parseLong( p.getProperty( "parser.sleep", "30000" ) );
	}
	catch( Exception e )
	{
		sleepTimeInMillis = 30000;
	}
	
	processedDir = p.getProperty( "parser.processedDir", null );

	if ( processedDir != null )
	{
		procDir = new File( processedDir  );
		procDir.mkdirs();
	}
	
	serverHost = p.getProperty( "parser.serverHost" );
	
	try
	{
		serverPort = Integer.parseInt( p.getProperty( "parser.serverPort", "5432" ) );
	}
	catch( Exception e )
	{
		serverPort = 5432;
	}

	loadDirectory = p.getProperty( "parser.directory" );

	try
	{
		debug = Boolean.valueOf( p.getProperty( "parser.debug", "false" ) ).booleanValue();
	}
	catch( Exception e )
	{
		debug = false;
	}

	try
	{
		processing = Boolean.valueOf( p.getProperty( "parser.processing", "true" ) ).booleanValue();
	}
	catch( Exception e )
	{
		processing = true;
	}
	PlayerSource.processing = processing;
	GameSource.processing   = processing;
	MapSource.processing    = processing;
	
	
}
/**
 * Insert the method's description here.
 * Creation date: (8/13/00 11:03:56 AM)
 * @param args java.lang.String[]
 */
public static void main(String[] args) 
{
	SimpleDateFormat sdf = new SimpleDateFormat( "MM/dd/yyyy - hh:mm:ss" );
	
	Date start = new Date();
	System.out.println( "Run Starting: " + sdf.format( start ) );
	
	Date end;
	ParserThread pt = new ParserThread();
	int fileListOffset = 0;

	String parserFile = System.getProperty( "parser.properties" );
	
	if ( parserFile != null )
			pt.loadParserProperties( parserFile );
	else
	{
		System.out.println( "FATAL ERROR! NO properties file detected!" );
		System.exit( -1 );
	}

	//
	// Connect to the DB
	//	
	try
	{
		String dbURL = DataSource.createDbUrl( dbHost, dbName, dbPort, null );
		DataSource.connectToDB( dbURL, dbLogin, dbPasswd );
	}
	catch( Exception e )
	{
		e.printStackTrace();
		System.exit( 0 );
	}

	//
	// Sort out the ServerRecord
	//
	ServerRecord sr = null;
	
	// ServerRecord not cached
	try
	{
		List l = DataManager.fetchLimited( ServerRecord.class, "address", serverHost );
		Iterator i = l.iterator();
		while ( i.hasNext() )
		{
			sr = (ServerRecord)i.next();
			if ( sr.getPort() == serverPort )
				break;
			else
				sr = null;
		}
	}
	catch( Exception e )
	{
		e.printStackTrace();
	}
	if ( sr == null )
	{
		// no ServerRecord in DB...create a new one
		sr = new ServerRecord();
		sr.setAddress( serverHost );
		sr.setPort( serverPort );
		// store in DB
		try
		{
			// this is weird and hacked, I know, but it gets me around a
			// postgres-specific limitation to my persistence code
			// until I can fix it properly
			DataManager.save( sr );
			sr.setObjectState( ObjectState.DIRTY );
			DataManager.save( sr );
		}
		catch( Exception ee )
		{
			ee.printStackTrace();
		}
	}				
		
	pt.setServerRecord( sr );


	if ( daemonMode && loadDirectory != null )
	{
		Thread t = new Thread( pt );
		t.start();
	}
	else
	{		
		if ( loadDirectory != null )
		{
			File fd = new File( loadDirectory );
			pt.loadDirectory( fd );

			end = new Date();
			
			System.out.println( "Run Started: " + sdf.format( start ) );				
			System.out.println( "Run complete: " + sdf.format( end ) );
			System.out.println( "\n\nprocessed " + (filesLoaded+filesSkipped+filesErrored) + " Files:\n\tLoaded " + 
					filesLoaded + "\n\tSkipped Files: " + filesSkipped );
			System.out.println( "\tFiles Errored: " + filesErrored );
			System.exit( 0 );
				
		}
	}
	end = new Date();
	System.out.println( "Run complete: " + sdf.format( end ) );
	System.out.println( "\n\nprocessed " + (filesLoaded+filesSkipped) + " Files:\n\tLoaded " + filesLoaded + "\n\tSkipped Files: " + filesSkipped );
	
}
/**
 * Insert the method's description here.
 * Creation date: (8/12/00 9:03:53 PM)
 * @param line java.lang.String
 */
public void parse( String line ) 
{
	if ( line.length() == 0 || line.charAt( 0 ) != 'L' )
	{
		if ( debug )
			System.out.println( "Not processed: " + line );
		return;
	}

	// deal with people talking separately
	if ( listenerList.size() > 0 && line.charAt( 25 ) == 2 )
	{
		parseSay( line.substring( 26 ) );
		return;
	}

	try
	{
		String event = line.substring( 25 );	
	
		//
		// Events that don't start with a "
		//	
		if ( event.charAt( 0 ) != '"' )
		{
			//
			// People rescuing/killing hostages
			//
			if ( event.endsWith( "hostage" ) )
			{
				GamePlayer gp;
				char c = event.charAt( event.length() - 16 );
				if ( c == 'o' )
				{
					String name = event.substring( 0, event.length() - 18 );
						
					gp = g.getPlayer( name );
					if ( gp != null )
						gp.addHostageTouch();
				}
				else if ( c == 'e' )
				{
					String name = event.substring( 0, event.length() - 22 );
						
					gp = g.getPlayer( name );
					if ( gp != null )
						gp.addHostageRescue();
				}
				else if ( c == 'k' )
				{
					// not there, which is weird, b/c I thought it was...
					String name = event.substring( 0, event.length() - 19 );
					gp = g.getPlayer( name );
					if ( gp != null )
						gp.addHostageKill();
				}
			}
			//
			// People planting & defusing the bomb
			//
			else if ( event.endsWith( "bomb" ) )
			{
				GamePlayer gp;
					
				char c = event.charAt( event.length() - 16 );
				if ( c == 'p' )
				{
					String name = event.substring( 0, event.length() - 17 );
					if ( debug )
						System.out.println( "'" + name + "' planted the bomb" );
					
					gp = g.getPlayer( name );
					if ( gp == null )
					{
						;
					}
					if ( gp != null )
						gp.addBombPlant();
				}
				else if ( c == 'd' )
				{
					String name = event.substring( 0, event.length() - 21 );
					if ( debug )
						System.out.println( "'" + name + "' defused the bomb" );
					
					gp = g.getPlayer( name );
					if ( gp != null )
						gp.addBombDefuse();
				}
			}
			//
			// TK!!!!!
			//
			else if ( event.endsWith( "teammate" ) )
			{
				String name = event.substring( 0, event.length() - 18 );
				
				if ( debug )
					System.out.println( "TK by '" + name + "'" );
						
				GamePlayer gp = g.getPlayer( name );
					if ( gp != null )
						gp.addTk();
				}
			
			else if ( event.startsWith( "Rcon" ) )
			{
				if ( event.indexOf( "status" ) != 0 )
					return;

				try
				{
					// cmd is now just the 1st char of the passwd up to the end of the last command
					String cmd = event.substring( event.lastIndexOf( '(' ) + 7, event.length() - 3 );

					int lindex = cmd.lastIndexOf( '"' );
					String idStr = cmd.substring( lindex + 1 );
					
					Date d = sdf.parse( line.substring( 2, 23 ) );

					// we now have the target (person kicked/banned)
					if ( idStr.charAt( 0 ) == '#' )
						idStr = idStr.substring( 1 );

					int aIndex = cmd.indexOf( '"' ) + 3;
					String action = cmd.substring( aIndex );
					if ( action.charAt( 0 ) == 'k' )
					{
						g.addKick( Integer.parseInt( idStr ), d );
					}
					else if ( action.charAt( 0 ) == 'b' )
					{
						if ( idStr.equals( "kick" ) )
						{
							idStr = cmd.substring( 0, cmd.length() - 7 );
							idStr = idStr.substring( idStr.lastIndexOf( '"' ) + 1 );
						}
						String minStr = action.substring( 8 );
						minStr = minStr.substring( 0, minStr.indexOf( '"' ) );
						
						int numMinutes = Integer.parseInt( minStr );
						g.addBan( Long.parseLong( idStr ), numMinutes, d );
					}
					else if ( action.charAt( 0 ) == 'a' )
					{
						if ( debug )
							System.out.println( "addip ban" );
						String ip = "";
						int numMinutes = 0;
						g.addBan( ip, numMinutes, d );
					}
				}
				catch( Exception e )
				{
					System.out.println( "rcon processing failed: " + e.getMessage() );
					
				}
			}		
			//
			// People killing the VIP
			//
			else if ( event.endsWith( "VIP" ) )
			{
				String name = event.substring( 0, event.length() - 20 );
				if ( debug )
					System.out.println( "'" + name + "' killed the VIP" );
				
				GamePlayer gp = g.getPlayer( name );
				if ( gp != null )
					gp.addVipKill();
			}		
			//
			// People joining/changing teams
			//		
			else if ( event.endsWith( "TERRORIST force" ) )
			{
				String playerName = event.substring( 0, event.length() - 31 );
				g.setCurrentTeam( playerName, GamePlayer.TERRORIST );
/*				
				GamePlayer p = g.getPlayer( playerName );
				if ( p != null )
				{
//					p.setCurrentTeam( GamePlayer.TERRORIST );
					g.setCurrentTeam( p.getWonID(), GamePlayer.TERRORIST );
				}
*/				
			}
			else if ( event.endsWith( "CT force" ) )
			{
				String playerName = event.substring( 0, event.length() - 24 );
				g.setCurrentTeam( playerName, GamePlayer.COUNTERTERRORIST );
/*				
				GamePlayer p = g.getPlayer( playerName );
				if ( p != null )
				{
//					p.setCurrentTeam( GamePlayer.COUNTERTERRORIST );
					g.setCurrentTeam( p.getWonID(), GamePlayer.COUNTERTERRORIST );
				}
*/				
			}
			else if ( event.startsWith( "Log file" ) )
			{
				// This event is not strictly necessary--a new Date() is created at
				// the time of object creation, but this will overwrite that with
				// the localtime from the server (as recorded in the log file)
				try
				{
					g = new Game( sdf.parse( line.substring( 2, 23 ) ) );
					g.setServerRecord( serverRecord );
				}
				catch( java.text.ParseException pe )
				{
					pe.printStackTrace();
					g = new Game();
				}
			}
			else if ( event.startsWith( "Spawn" ) )
			{
				String mapName = event.substring( 17, ( event.length() - 1 ) );
				if ( debug )
					System.out.println( "mapName: " + mapName );
				g.setMapName( mapName );
			}
	/*		
			else if ( event.startsWith( "Map CRC" ) )
			{
				String mapCRC = event.substring( 9, ( event.length() - 2 ) );
				g.getMap().setCrc( Long.parseLong( mapCRC ) );
			}
	*/		
			else if ( event.startsWith( "Log closed." ) )
			{
				// We're ending this game
				try
				{
					if ( processing )
					{
						g.setEndTime( sdf.parse( line.substring( 2, 23 ) ) );
						if ( summary )
							printSummary( g );
						if ( GameSource.save( g ) )
							filesLoaded++;
						else
							filesSkipped++;
					}
				}
				catch( Exception pe )
				{
					System.out.println( "Failed to parse time: " + line.substring( 2, 23 ) );
					System.out.println( "error: " + pe.getMessage() );
					g.setEndTime( new Date() );
				}
			}
			else if ( event.startsWith( "Log file started." ) )
			{
				// We're starting a new game
				try
				{
					// make sure the old game was saveed
					if ( g != null && g.getObjectState() == ObjectState.NEW )
						GameSource.save( g );
						
					g = new Game();
					g.setStartTime( sdf.parse( line.substring( 2, 23 ) ) );
				}
				catch( java.text.ParseException pe )
				{
					pe.printStackTrace();
				}
			}
			// figure out who won:  
			//   ***COUNTER, **BOMB Defused
			//   ***TERR, ***Target Successfully bombed
			//   ***Round DRAW
			else if ( event.charAt( 0 ) == '*' )
			{
				char c = event.charAt( 3 );
				if ( c == 'T' )
					g.addWin( Game.TERR );
				else if ( c == 'C' || c =='B' )
					g.addWin( Game.CT );
				else if ( c == 'R' )
					g.addWin( Game.DRAW );
			}
				
		}
		else if ( event.charAt( 0 ) == '"' )
		{
			//
			// The event starts with a "
			int lindex = event.indexOf( "\"", 1 );
			// get the quoted string
			String actor = event.substring( 1, lindex );
			String nextWord = event.substring( lindex + 2 );		

			//
			// someone kills someone:
			// "eminem<421>" killed "[TERRORIST] [DS]Neo<422>" with m4a1
			//
			if ( nextWord.startsWith( "killed" ) )
			{
	/*
				Frag f = new Frag();
	*/
				char k = nextWord.charAt( 7 );
				if ( k != '"' )
				{
					if ( debug)
						System.out.println( "skipping: " + event );
					return;
				}
/*				
				if ( nextWord.endsWith( "world" ) )
				{
					// the player changed teams and the next line will be
					// them joining a team
					return;
				}
				if ( nextWord.indexOf( "worldspawn" ) > 0 )
				{
					// the person fell
					return;;
				}
*/				
				lindex = actor.lastIndexOf( '<' );
				String actorServerId = actor.substring( lindex + 1, actor.length() - 1 );

//				String victim = nextWord.substring( nextWord.indexOf( '"' ) + 1, nextWord.lastIndexOf( '"' )	 );
				String victim = nextWord.substring( 8, nextWord.lastIndexOf( '"' )	 );

				int vindex = victim.lastIndexOf( '<' );
				String victimServerId = victim.substring( vindex + 1, victim.length() - 1 );
								
				int serverId = Integer.parseInt( actorServerId );
				
				if ( debug )	
					System.out.println( serverId + " killed " + victimServerId );
					
				GamePlayer killer = g.getPlayer( serverId );
				
				if ( killer == null ) // not likely
				{
					// try again with their name
					String kStr = actor.substring( 0, lindex - 1 );
					killer = g.getPlayer( kStr );
				}
				
				if ( killer != null )
					killer.addKill();
				else
					if ( debug )
						System.out.println( "No record of killer" );

				int victimId = Integer.parseInt( victimServerId );
				GamePlayer v = g.getPlayer( victimId );
				if ( v == null )
				{
					// try again with their name
					String vStr = victim.substring( victim.indexOf( ']' )+ 2, vindex );
					v = g.getPlayer( vStr );
				}
				if ( v != null )
					v.addDeath();
				else
					if ( debug )	
						System.out.println( "No record of victim" );
					
				
	/*
				if ( victim.startsWith( "[CT]" ) )
				{
	//				f.setVictimTeam( bing.cstrike.GamePlayer.CT );
					System.out.println( "victim was a CT" );
				}
				else
				{
	//				f.setVictimTeam( bing.cstrike.GamePlayer.TERRORIST );
					System.out.println( "victim was a Terr" );
				}

				String weapon = nextWord.substring( nextWord.lastIndexOf( ' ' ) );

	//			f.setWeapon( weapon );
				System.out.println( "weapon: " + weapon );
	*/

				}
			//
			// Someone connects to the server
			// "adam_beyer<424><WON:7794908>" connected, address "213.3.10.28:27005"
			//
			else if ( nextWord.startsWith( "connected," ) )
			{
				// actor is  adam_beyer<424><WON:7794908>
				// nextWord is connected, address "213.3.10.28:27005"
				int index = actor.lastIndexOf( '<' );
				String wonIDStr = actor.substring( index + 5, actor.length() - 1 );
				
				String subStr = actor.substring( 0 , index - 1 );

				int nameEnd = subStr.lastIndexOf( '<' );
				String serverIdStr = subStr.substring( nameEnd + 1 );

				String playerName = actor.substring( 0, nameEnd );
				String playerIP = nextWord.substring( nextWord.indexOf( '"' ) + 1, nextWord.length() - 1 );	

				
				if ( debug )
					System.out.println( wonIDStr + "\t" +	serverIdStr + "\t" + playerName + "\t" + playerIP );

				// we have the following info about our new player at this point:
				// wonId, serverId, name, and IP
				int serverId = Integer.parseInt( serverIdStr );
				long wonId = Long.parseLong( wonIDStr );
				g.addPlayer( playerName, wonId, serverId, playerIP );
				g.addAlias( serverId, playerName );
			}
			/*	
			 * just let it grow for now, since it gets reset on server restart and also,
			 * changelevel tends to cause everyone to disconnect, but then enter the game normally
			 * when the next map displays, so it can create data problems
			 */
	 /*
			else if ( nextWord.startsWith( "discon" ) )
			{
				/*
				 * remove ServerToWon
				 *
				int index = actor.lastIndexOf( '<' );
				String serverIDStr = actor.substring( index + 1, actor.length() - 1 );
				if ( processing )			 
					MapSource.removeMapping( Integer.parseInt( serverIDStr ), serverRecord.getServerId() );
			}
		*/	
			else if ( nextWord.startsWith( "has enter" ) )
			{
				int index = actor.lastIndexOf( '<' );			
				String playerName = actor.substring( 0 , index );
				String serverIdStr = actor.substring( index + 1, actor.length() - 1 );

				if ( debug )
					System.out.println( "Entered game: playerName: '" + playerName + "'\t" + "\tserverId: " + serverIdStr );

				g.addAlias( Integer.parseInt( serverIdStr), playerName );
			}
			else if ( nextWord.startsWith( "chang" ) )
			{
				int index = actor.lastIndexOf( '<' );			
				String serverIdStr = actor.substring( index + 1, actor.length() - 1 );
				
				String newName = nextWord.substring( 17, nextWord.length() - ( serverIdStr.length() + 3 ) );
				int sId = Integer.parseInt( serverIdStr );
				GamePlayer gp = g.getPlayer( sId );
				if ( gp != null )
					gp.addAlias( newName );	
				g.addAlias( sId, newName )	;
			}
					
			// This should be the last check in the starts-with-" block
			else if ( event.startsWith( "\"mp_friendlyfire\"" ) )
			{
				g.setFriendlyFireOn( ( event.charAt( 21 ) == '1' ) );
				if ( debug )
					System.out.println( "friendly fire on: " + g.isFriendlyFireOn() );
			}
			else if ( event.startsWith( "\"mp_c4timer\"" ) )
			{
				g.setC4Timer( Integer.parseInt( event.substring( 16, event.lastIndexOf( '"' ) ) ) ); 
				if ( debug )
					System.out.println( "c4 timer : '" + g.getC4Timer() + "'" );
			}
		}
		say( event );	
	}
	catch( Exception e )
	{
		System.out.println( "Failed to parse: " + line );
		e.printStackTrace();
	}
}
/**
 * Determine who ( ct/t && alive/dead ) said something
 * and echo it to any listening consoles
 * Creation date: (8/15/00 10:23:36 AM)
 * @param packet java.net.DatagramPacket
 */
private void parseSay( String talk )
{
	if ( talk.charAt( 0 ) == '*' )
	{
		// saying dead
		if ( talk.charAt( 6 ) == '(' )
		{
			// dead say_team
			if ( talk.charAt( 7 ) == 'T' )
			{
				// terr
				say( talk, SAY_DEAD_T );
			}
			else if ( talk.charAt( 7 ) == 'C' )
			{
				// ct
				say(  talk, SAY_DEAD_CT );
			}
			else
				// just a name that starts w/ '('
				say( talk, SAY_DEAD );
		}
		else
		{
			// dead say
			say( talk, SAY_DEAD );
		}
	}
	else if ( talk.charAt( 0 ) == '(' )
	{
		// alive say_team
		if ( talk.charAt( 1 ) == 'T' )
		{
			say( talk, SAY_ALIVE_T );
		}
		else
		{
			say( talk, SAY_ALIVE_CT );
		}
	}
	else
	{
		// alive say
		say( talk, SAY_ALIVE );
	}
}
/**
 * Print a summary of the score, map, and players
 * in a given game
 * Creation date: (9/1/00 8:15:02 AM)
 * @param g org.halfcat.halflife.cstrike.Game
 */
public static void printSummary(Game g) 
{
	System.out.println( "Map: " + g.getMapName() + "\tCT Wins: " + g.getCtWins() + "\tTerr Wins: " + g.getTerrWins() );
	Hashtable players = g.getGamePlayers( GamePlayer.UNDEFINED );
	System.out.println( "Unknown: " );
	Iterator ii = players.keySet().iterator();
	while ( ii.hasNext() )
	{
		Object key = ii.next();
		GamePlayer gp = (GamePlayer)players.get( key );
		System.out.println( key + ": \"" + gp.getCurrentName() + "\"\t" + 
			gp.getKills() + "/" + gp.getDeaths() + " -- " + 
			gp.getTouchedHostages() + "/" + gp.getRescuedHostages() + "/" + gp.getKilledHostages() +
			" -- " + gp.getBombPlants() + "/" + gp.getBombDefuses() + 
			" -- " + gp.getReconnects() );
	}


	System.out.println( "\nCounter-Terrorists: " );
	players = g.getGamePlayers( GamePlayer.COUNTERTERRORIST );
	ii = players.keySet().iterator();
	while ( ii.hasNext() )
	{
		Object key = ii.next();
		GamePlayer gp = (GamePlayer)players.get( key );
		System.out.println( key + ": \"" + gp.getCurrentName() + "\"\t" + 
			gp.getKills() + "/" + gp.getDeaths() + " -- " + 
			gp.getTouchedHostages() + "/" + gp.getRescuedHostages() + "/" + gp.getKilledHostages() +
			" -- " + gp.getBombPlants() + "/" + gp.getBombDefuses() + 
			" -- " + gp.getReconnects() );
	}	

	System.out.println( "\nTerrorists: " );
	players = g.getGamePlayers( GamePlayer.TERRORIST );
	ii = players.keySet().iterator();
	while ( ii.hasNext() )
	{
		Object key = ii.next();
		GamePlayer gp = (GamePlayer)players.get( key );
		System.out.println( key + ": \"" + gp.getCurrentName() + "\"\t" + 
			gp.getKills() + "/" + gp.getDeaths() + " -- " + 
			gp.getTouchedHostages() + "/" + gp.getRescuedHostages() + "/" + gp.getKilledHostages() +
			" -- " + gp.getBombPlants() + "/" + gp.getBombDefuses() + 
			" -- " + gp.getReconnects() );
	}	
	
	
}
/**
 * Insert the method's description here.
 * Creation date: (8/15/00 8:50:34 AM)
 */
private void printUsage( String progName ) 
{
	System.out.println( "Usage: java -Djdbc.drivers=org.postgresql.Driver org.halfcat.halflife.ParserThread -daemon -logDir <full_path_to_logs_dir>" );
}
/**
 * Insert the method's description here.
 * Creation date: (8/15/00 8:42:36 AM)
 * @param args java.lang.String[]
 */
protected int processCmdLine( String[] args ) 
{
	
	// Parse the command-line
	int argCount = 0;
	int x = 0;
	while ( x < args.length )
	{
		if ( args[x].equals( "-logDir" ) )
		{
			loadDirectory = args[++x];
		}		
		else if ( args[x].equals( "-daemon" ) )
		{
			daemonMode = true;
		}
		else
		{
			printUsage( args[0] );
			System.exit( 0 );
		}
		x++;		
	}
	return argCount;	
}
/**
 * Insert the method's description here.
 * Creation date: (9/1/00 7:36:33 AM)
 * @param f java.io.File
 * @param pt org.halfcat.halflife.ParserThread
 */
private void processFile( File f, ParserThread pt, boolean moveFiles ) 
{
	try
	{
		BufferedReader fr = new BufferedReader( new FileReader( f ) );
		System.out.println( "processing " + f.getAbsoluteFile() + "..." );
		
		while ( fr.ready() )
			pt.parse( fr.readLine() );

		fr.close();
			
		System.out.println( "Loaded." );
		
		if ( moveFiles && processedDir != null )
		{
			// move the file to a processed dir
			
			File procFile = new File( processedDir + f.separator + f.getName() );
			if ( ! procDir.isDirectory() )
			{
				System.out.println( "WARNING: " + processedDir + " is NOT a directory!  Files will not be moved" );
				
				if ( daemonMode )
				{
					System.out.println( "FATAL ERROR:  daemonMode cannot run in this configuration...exiting" );
					System.exit( -1 );
				}
			}
			else
			{
				System.out.print( "Moving to " + procFile.getAbsolutePath() + "..." );
				if ( f.renameTo( procFile ) )
					System.out.println( "Succeeded" );
				else
					System.out.println( "Failed" );
			}
		}
		else
		{
			System.out.println( "Not moving files" );
		}
		printSummary( g );
	}
	catch( FileNotFoundException fnfe )
	{
		System.out.println( "No such file: " + f.getAbsolutePath() );
	}
	catch( IOException ioe )
	{
		ioe.printStackTrace();
	}	
}
/**
 * Insert the method's description here.
 * Creation date: (9/1/00 8:38:01 AM)
 */
public void run() 
{
	System.out.println( "Running in daemon Mode" );
	boolean running = true;
	
	File fd = new File( loadDirectory );
	long lastlastRun;
	long lastRun = 1;
	int processDirSize = 0;
	
	// run in daemon mode
	while ( running )
	{
		File[] fa = fd.listFiles();
		if ( fa.length != processDirSize )
		{
			processDirSize = fa.length;
			lastlastRun = lastRun;
			lastRun = new Date().getTime();
			for ( int x = 0; x < processDirSize; x++ )
			{
				if ( ! fa[x].isDirectory() && fa[x].length() > 0  && fa[x].lastModified() >= lastlastRun )
					processFile( fa[x], this, true );
			}
//			lastRun = new Date().getTime();
		}

		
		try
		{
			Thread.currentThread().sleep( sleepTimeInMillis );
			Thread.currentThread().yield();
		}
		catch( InterruptedException ie )
		{
			System.out.println( "Interrupted" );
		}
	}
}
/**
 * Insert the method's description here.
 * Creation date: (8/26/00 7:41:28 AM)
 * @param msg java.lang.String
 * @param msgType int
 */
public void say( String msg ) 
{
	if ( listenerList.size() == 0 )
		return;
		
	Iterator i = listenerList.iterator();
	Writer w;

	msg = msg + "<br>";
	while ( i.hasNext() )
	{
		w = (Writer)i.next();
		try
		{
			
			w.write( msg );
			w.flush();
		}
		catch( IOException ioe )
		{
			if ( debug )
			{
				System.out.println( "write failed for Writer" );
			}
			
			i.remove();
		}
	}
	
}
/**
 * Create a formated, color-ized HTML message
 * for console use
 * Creation date: (8/26/00 7:41:28 AM)
 * @param msg java.lang.String
 * @param msgType int
 */
public void say( String msg, int msgType ) 
{
	int c = msg.indexOf( ':' );
	
	String name = msg.substring( 0, c );
	String text = msg.substring( c + 1 );
	
	StringBuffer sb = new StringBuffer();
	sb.append( "<font color=\"" );
	sb.append( textColors[msgType] );
	sb.append( "\"><b>" );
	sb.append( name );
	sb.append( "</b> " );
	sb.append( text );	
	sb.append( "</font>" );

	say( sb.toString() );
	
}
/**
 * Insert the method's description here.
 * Creation date: (8/10/00 4:26:22 PM)
 * @param newIdString java.lang.String
 */
public void setIdString(java.lang.String newIdString) 
{
	idString = newIdString;
}
/**
 * If true, then ParserThread is loading data to the DB
 * Creation date: (9/1/00 6:55:48 AM)
 * @param newProcessing boolean
 */
public static void setProcessing( boolean newProcessing ) 
{
	if ( debug )
	{
		if ( newProcessing )
			System.out.println( "Loading data to the Database" );
		else
			System.out.println( "Monitoring, but not loading" );
	}
	processing = newProcessing;
}
/**
 * Insert the method's description here.
 * Creation date: (8/30/00 8:41:37 AM)
 * @param newServerRecord org.halfcat.halflife.ServerRecord
 */
public void setServerRecord(ServerRecord newServerRecord) 
{
	serverRecord = newServerRecord;
	g.setServerRecord( newServerRecord );
}
/**
 * Insert the method's description here.
 * Creation date: (9/3/00 12:44:18 PM)
 * @param newSleepTimeInMillis long
 */
public void setSleepTimeInMillis(long newSleepTimeInMillis) {
	sleepTimeInMillis = newSleepTimeInMillis;
}
}
