package org.halfcat.halflife;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import java.math.*;

/**
 * Insert the type's description here.
 * Creation date: (8/18/00 5:54:01 PM)
 * @author: Administrator
 */
public class StatusServer
	extends Server
{
	

	//used for reporting	
	String game;
	// Used by Half-Life, at least
	String longGame;

	List players = new ArrayList();


/**
 * Construct a new server
 */
public StatusServer() 
{
	super();
}
/**
 * Server constructor comment.
 */
public StatusServer( String qAddress, int qPort )
	throws
		IOException,
		SocketException
{
	super( qAddress, qPort );
//	refresh();
}
/**
 * Return the game/mod type--"valve", "tfc", "cstrike", etc
 * Creation date: (8/19/00 7:39:06 AM)
 * @return java.lang.String
 */
public String getGame() 
{
	return game;
}
/**
 * Insert the method's description here.
 * Creation date: (8/19/00 7:51:03 AM)
 * @return java.lang.String
 */
public java.lang.String getLongGame() {
	return longGame;
}
/**
 * Insert the method's description here.
 * Creation date: (8/19/00 7:39:29 AM)
 * @return java.util.List
 */
public List getPlayers() 
{
	try
	{
//		players = parsePlayers( queryServer( HL_PLAYERS, queryAddress, queryPort ) );
		players = parsePlayers( sendCommand( "players", queryAddress, queryPort ) );
	}
	catch( Exception e )
	{
		System.out.println( e.getMessage() );
		return new ArrayList();
	}
	
	return players;
}
/**
 * Insert the method's description here.
 * Creation date: (8/25/00 10:13:00 AM)
 * @param args java.lang.String[]
 */
public static void main(String[] args) 
{
	try
	{
		StatusServer ss = new StatusServer();
		ss.setQueryAddress( "192.168.1.7" );
		ss.setQueryPort( 27017 );
		ss.refresh();
		System.out.println( ss.getServerName() );
		List playerList = ss.getPlayers();
		Iterator i = playerList.iterator();
		while ( i.hasNext() )
		{
			Player p = (Player)i.next();
			System.out.println( p.getNumber() + "\t" + p.getPlayerName() + "\t" + p.getFrags() );
		}
	}
	catch( Exception e )
	{
		e.printStackTrace();
	}
	
}
/**
 * Parse the info header return.  Data comes back:
 * 'C' \0 ip:port 
 */
private void parseInfo( byte[] info )
{
	if ( info == null )
	{
		if ( debug )
			System.out.println( "StatusServer.parseInfo( null )" );
		return;
	}
	numPlayers = (int)info[info.length-3];
	maxPlayers = (int)info[info.length-2];
	StringTokenizer st = new StringTokenizer( new String( info ), "\0" );

	// parse the ip:port into ip and port
	String tmpAddress = st.nextToken().substring( 1 );
	int colonIndex = tmpAddress.indexOf( ':' );
	String tmpPort = tmpAddress.substring( colonIndex + 1 );
	try
	{
		this.port = Integer.parseInt( tmpPort );
	}
	catch( NumberFormatException pe )
	{
		System.out.println( "Failed to parse " + tmpPort );
		this.port = 27015;
	}

	this.address = tmpAddress.substring( 0, colonIndex );
	
	this.serverName = st.nextToken();
	this.mapName = st.nextToken();
	this.game = st.nextToken();
	this.longGame = st.nextToken();

}
/**
 * Insert the method's description here.
 * Creation date: (8/19/00 7:59:36 AM)
 * @param buf byte[]
 * @exception java.io.IOException The exception description.
 */
private List parsePlayers( byte[] buf ) 
	throws java.io.IOException 
{
	List al = new ArrayList();


	// 'D' (ASCII 68) is the first char of a player message
	if ( buf[0] != 68 )
		throw new IOException( "Invalid message header" );	
	
	// number of players is stored in the first byte after the header
	int numPlayers = buf[1];
	
	Player p;
	int bufIndex = 2;
	for ( int x = 0; x < numPlayers; x++ )
	{
		p = new Player();

		// set the number
		p.setNumber( buf[bufIndex++] );

		// set the name
		StringBuffer sb = new StringBuffer();
		while ( buf[bufIndex] != 0 )
			sb.append( (char)buf[bufIndex++] );
		p.setPlayerName( sb.toString() );

		p.setFrags( buf[++bufIndex] );		
		bufIndex++;
		
		al.add( p );
		if ( debug )
			System.out.print( "Chars for (" + p.getNumber() + ") " + p.getPlayerName() + "(" + p.getFrags() + " frags):" );

		int seconds = 0;
		byte[] time = new byte[4];
		int index = 0;
		time[index++] = buf[bufIndex + 1];
		time[index++] = buf[bufIndex + 2];
		time[index++] = buf[bufIndex + 3];
		time[index++] = buf[bufIndex + 4];
		int timeIndex = 0;
		
		
		if ( debug )
		{
			for ( x = 0; x < 7; x++ )
			{
				System.out.print( " " + (int)buf[bufIndex] + "/" + (char)buf[bufIndex] );
				System.out.print( "/" + Integer.toHexString( buf[bufIndex] ) );
				bufIndex++;
			}
			System.out.println("");
		
			BigInteger bd = new BigInteger( time );
			System.out.println("Time in seconds: " + bd.intValue() );
		}
		else
			bufIndex += 7;

	}	

	if ( debug )
	{	
		// display response
		int buflen = buf.length;
		System.out.println("Received " + buflen + " bytes back." );
		for (int x = 0; x < buflen; x++)
			System.out.print(" " + (int) buf[x]);
		System.out.println("");

		// print the characters
		for (int x = 0; x < buflen - 1 ; x++)
		{
			char c;
			if (buf[x] != (byte) 0)
				c = (char) buf[x];
			else
				c = ' ';
			System.out.print(c);
		}
		System.out.println("");
	}

	return al;
}
/**
 * Insert the method's description here.
 * Creation date: (8/20/00 1:40:29 PM)
 */
public void refresh() 
	throws
		IOException,
		SocketException
{
//	parseInfo( queryServer( HL_INFO, queryAddress, queryPort ) );
	if ( debug )
		System.out.println( "StatusServer refreshing " + queryAddress + ":" + queryPort );
	parseInfo( sendCommand( "info", queryAddress, queryPort ) );
}
/**
 * Insert the method's description here.
 * Creation date: (8/19/00 7:39:06 AM)
 * @param newGame java.lang.String
 */
public void setGame(java.lang.String newGame) {
	game = newGame;
}
/**
 * Insert the method's description here.
 * Creation date: (8/19/00 7:51:03 AM)
 * @param newLongGame java.lang.String
 */
public void setLongGame(java.lang.String newLongGame) {
	longGame = newLongGame;
}
/**
 * Insert the method's description here.
 * Creation date: (8/19/00 7:39:29 AM)
 * @param newPlayers java.util.ArrayList
 */
private void setPlayers(java.util.ArrayList newPlayers) {
	players = newPlayers;
}
}
