package org.halfcat.halflife.cstrike;

import org.halfcat.halflife.ServerToWon;
import java.sql.SQLException;
import java.net.*;
import java.util.*;
import org.halfcat.persistence.*;
import org.halfcat.halflife.MapSource;
import org.halfcat.halflife.ServerRecord;
/**
 * Insert the type's description here.
 * Creation date: (8/13/00 11:45:55 AM)
 * @author: Administrator
 */
public class Game
	extends org.halfcat.persistence.AbstractPersistable
{
	private java.util.Date startTime;
	private java.util.Date endTime = null;

	private String mapName = null;
	private boolean friendlyFireOn = false;
	int c4Timer = 0;

	int ctWins   = 0;
	int terrWins = 0;
	int draws    = 0;

	// round winners
	public static final int CT   = 0;
	public static final int TERR = 1;
	public static final int DRAW = 2;
	

	private Hashtable gamePlayers = new Hashtable();
	private Hashtable nameSource  = new Hashtable();
	private Hashtable wonSource   = new Hashtable();

	// array of hashtables, indexed by CT, TERR
	private Hashtable[] playerByTeam = new Hashtable[] { new Hashtable(), new Hashtable(), new Hashtable() };

	private ServerRecord serverRecord;
	private int serverId = 0;

	private List kickBans = null;
/**
 * Game constructor comment.
 */
public Game() 
{
	super();
}
/**
 * Game constructor comment.
 */
public Game( Date start ) 
{
	super();
	startTime = start;
}
/**
 * Add a mapping of a player's alias to ther serverId
 * Creation date: (8/29/00 1:21:59 PM)
 * @param serverId int
 * @param alias java.lang.String
 */
public void addAlias( int serverId, String alias ) 
{
	GamePlayer gp = getPlayer( serverId );
	if ( gp != null )
	{
		nameSource.put( alias, gp );
		gp.setCurrentName( alias );
	}
	else
	{
		// if we don't have a gamePlayer for this serverId,
		// then we need to sort it out
		long wonId = MapSource.getMapping( serverId, serverRecord.getServerId() );
		if ( wonId < 0 )
		{
			System.out.println( "***unable to find wonID for player!--not mapping alias" );
			return;
		}
		
		addPlayer( alias, wonId, serverId, null );
		
		nameSource.put( alias, new Integer( serverId ) );
		gp = getPlayer( serverId );
		nameSource.put( alias, gp );
		gp.setCurrentName( alias );
	}
}
/**
 * add a record of Banning the specified wonId from the server
 * Creation date: (8/29/00 10:33:23 AM)
 */
public synchronized void addBan( long wonId, int numMinutes, Date d ) 
{
	KickBan kb = new KickBan();
	kb.setAction( KickBan.BANID );
	kb.setWonID( wonId );
	kb.setBanTime( numMinutes );
	kb.setDateTime( d );
	kb.setServer( serverId );

	if ( kickBans == null )
		kickBans = new ArrayList();

	kickBans.add( kb );

	GamePlayer gp = getPlayer( wonId );
	gp.addKickban();
}
/**
 * add a record of Banning the specified IP address from the server
 * TO-DO:  This method does not currently add a kickban entry to the player
 * being banned b/c there's no mapping of address to player at the moment
 * Creation date: (8/29/00 10:33:23 AM)
 */
public synchronized void addBan( String ipAddress, int numMinutes, Date d ) 
{
	KickBan kb = new KickBan();
	kb.setAction( KickBan.BANIP );
	kb.setBanTime( numMinutes );
	kb.setBanIP( ipAddress );
	kb.setDateTime( d );
	kb.setServer( serverId );

	if ( kickBans == null )
		kickBans = new ArrayList();

	kickBans.add( kb );
	
}
/**
 * Add gp to the Game's cache
 * Creation date: (8/28/00 2:35:48 PM)
 * @param gp org.halfcat.halflife.cstrike.GamePlayer
 */
public void addGamePlayer( GamePlayer gp ) 
{
	Integer i = new Integer( gp.getServerId() );

	gamePlayers.put( i, gp );
	wonSource.put( new Long( gp.getWonID() ), gp );

	playerByTeam[gp.getCurrentTeam()].put( new Long( gp.getWonID() ), gp );
	
}
/**
 * add a record of Kicking the specified wonId from the server
 * Creation date: (8/29/00 10:33:23 AM)
 * @param int serverId the id # assigned to the player being kicked by the server
 * @param java.util.Date The date/time the player was kicked at
 */
public synchronized void addKick( int serverId, Date d ) 
{
	// The two different serverId's here are pretty confusing, I know.
	// this.serverId is the objectId of the serverRecord of the server this player
	// is being kicked from.
	//
	// serverId in the params is the sequential id # assigned to the player when they
	// connected.
	
	KickBan kb = new KickBan();
	kb.setAction( KickBan.KICK );
	long wonId = MapSource.getMapping( serverId, this.serverId );
	kb.setWonID( wonId );
	kb.setDateTime( d );
	kb.setServer( serverId );

	if ( kickBans == null )
		kickBans = new ArrayList();

	kickBans.add( kb );

	GamePlayer gp = getPlayer( serverId );
	gp.addKickban();
}
/**
 * Add a new player to the Game
 * Creation date: (8/13/00 2:09:11 PM)
 * @param name java.lang.String
 * @param wonID long
 * @param serverId int
 * @param sourceIP java.lang.String
 */
public void addPlayer( String name, long wonID, int serverId, String sourceIP ) 
{
	// Create this mapping first thing
	MapSource.addMapping( serverId, wonID, serverRecord.getServerId() );

	GamePlayer gp;
	Player p;
	// see if we already have a reference to this player for this game
	Long l = new Long( wonID );
	if ( wonSource.containsKey( l ) )
	{
		gp = (GamePlayer)wonSource.get( l );
		
		if ( playerByTeam[gp.getCurrentTeam()].containsKey( l ) )
  		((GamePlayer)playerByTeam[gp.getCurrentTeam()].get( l )).addReconnect();
		else
			gp.addReconnect();
		
		Integer oldServerId = new Integer( gp.getServerId() );
		gamePlayers.remove( oldServerId );
		
		p = gp.getPlayer();
	}
	else
	{
		// Load the player, either from cache or from the DB
		p = PlayerSource.getPlayer( wonID );		
		if ( p == null )
		{
			// if the player doesn't exist, create a new one and save it to the DB
			p = new Player();
			p.setWonID( wonID );
			PlayerSource.addPlayer( p );
		}
		
		// otherwise, create a new one	
		gp = new GamePlayer();
		gp.setPlayer( p );
		gp.setServerId( serverId );
	}
	
	// do these in any case
	p.addAlias( name );
	p.addIP( sourceIP );
	
	gamePlayers.put( new Integer( serverId ), gp );
	wonSource.put( l, gp );
	gp.setCurrentName( name );	
}
/**
 * Insert the method's description here.
 * Creation date: (8/27/00 9:19:46 AM)
 * @param team int
 */
public void addWin( int team ) 
{
	if ( team == CT )
		ctWins++;
	else if ( team == TERR )
		terrWins++;
	else if ( team == DRAW )
		draws++;
	else
		System.out.println( "Unknown team #: " + team );
}
/**
 * Insert the method's description here.
 * Creation date: (9/3/00 6:42:26 AM)
 * @return org.halfcat.halflife.cstrike.GamePlayer
 * @param gp org.halfcat.halflife.cstrike.GamePlayer
 */
private GamePlayer getActivePlayer( GamePlayer gp ) 
{
	if ( gp == null )
		return null;
		
	Long w = new Long( gp.getWonID() );
	
	if ( playerByTeam[gp.getCurrentTeam()].containsKey( w ) )
		return (GamePlayer)playerByTeam[gp.getCurrentTeam()].get( w );
		
	return gp;		
}
/**
 * Insert the method's description here.
 * Creation date: (8/28/00 3:57:21 PM)
 * @return int
 */
public int getC4Timer() {
	return c4Timer;
}
/**
 * Insert the method's description here.
 * Creation date: (8/28/00 3:57:21 PM)
 * @return int
 */
public int getCtWins() {
	return ctWins;
}
/**
 * Insert the method's description here.
 * Creation date: (8/28/00 3:57:21 PM)
 * @return int
 */
public int getDraws() 
{
	return draws;
}
/**
 * Insert the method's description here.
 * Creation date: (8/13/00 11:47:19 AM)
 * @return java.util.Date
 */
public java.util.Date getEndTime() 
{
	return endTime;
}
/**
 * Insert the method's description here.
 * Creation date: (9/10/00 6:54:53 AM)
 * @return int
 */
public int getGameId() 
{
	return m_objectId;
}
/**
 * Insert the method's description here.
 * Creation date: (8/28/00 2:46:22 PM)
 * @return java.util.Hashtable
 */
public List getGamePlayers() 
{
	ArrayList al = new ArrayList();
	for ( int x = 0; x < playerByTeam.length; x++ )
	{
		al.addAll( playerByTeam[x].values() );
	}
	return al;
}
/**
 * Insert the method's description here.
 * Creation date: (8/28/00 2:46:22 PM)
 * @return java.util.Hashtable
 */
public Hashtable getGamePlayers( int team ) 
{
	try
	{
		return playerByTeam[team];
	}
	catch( Exception e )
	{
		e.printStackTrace();
		return wonSource;
	}
}
/**
 * Insert the method's description here.
 * Creation date: (9/9/00 7:32:04 AM)
 * @return java.util.List
 */
public java.util.List getKickBans() {
	return kickBans;
}
/**
 * Insert the method's description here.
 * Creation date: (8/13/00 11:51:54 AM)
 * @return bing.cstrike.GameMap
 */
public String getMapName() 
{
	return mapName;
}
/**
 * Insert the method's description here.
 * Creation date: (8/28/00 10:50:59 AM)
 * @return org.halfcat.halflife.cstrike.GamePlayer
 * @param serverId int
 */
public GamePlayer getPlayer( int serverId ) 
{
	GamePlayer gp = null;
	Integer i = new Integer( serverId );
	if ( gamePlayers.containsKey( i ) )
		gp = (GamePlayer)gamePlayers.get( i );
	else
	{
		Player p = PlayerSource.getPlayer( MapSource.getMapping( serverId, serverRecord.getServerId() ) );
		
		if ( p != null )
		{
			gp = new GamePlayer();
			gp.setServerId( serverId );
			gp.setPlayer( p );
			addGamePlayer( gp );
		}
	}
	return getActivePlayer( gp );
}
/**
 * Insert the method's description here.
 * Creation date: (8/28/00 10:50:59 AM)
 * @return org.halfcat.halflife.cstrike.GamePlayer
 * @param serverId int
 */
public GamePlayer getPlayer( long wonId ) 
{
	GamePlayer gp = null;
	Long l = new Long( wonId );
	if ( wonSource.containsKey( l ) )
		gp = (GamePlayer)wonSource.get( l );
	else
	{
		Player p = PlayerSource.getPlayer( wonId );
		
		if ( p != null )
		{
			gp = new GamePlayer();
			gp.setServerId( serverId );
			gp.setPlayer( p );
			addGamePlayer( gp );
		}
	}
	return getActivePlayer( gp );
}
/**
 * Insert the method's description here.
 * Creation date: (8/28/00 10:50:59 AM)
 * @return org.halfcat.halflife.cstrike.GamePlayer
 * @param currentName java.lang.String
 */
public GamePlayer getPlayer( String currentName ) 
{
	if ( nameSource.containsKey( currentName ) )
	{
		GamePlayer gp = (GamePlayer)nameSource.get( currentName );
		return getActivePlayer( gp );
	}
	else
		return null;
}
/**
 * Insert the method's description here.
 * Creation date: (8/30/00 12:30:56 PM)
 * @return int
 */
public int getServerId() 
{
	return serverId;
}
/**
 * Insert the method's description here.
 * Creation date: (8/30/00 12:03:25 PM)
 * @return ServerRecord
 */
public ServerRecord getServerRecord() {
	return serverRecord;
}
/**
 * Insert the method's description here.
 * Creation date: (8/13/00 11:47:08 AM)
 * @return java.util.Date
 */
public java.util.Date getStartTime() 
{
	return startTime;
}
/**
 * Insert the method's description here.
 * Creation date: (8/28/00 3:57:21 PM)
 * @return int
 */
public int getTerrWins() {
	return terrWins;
}
/**
 * Insert the method's description here.
 * Creation date: (8/13/00 12:03:00 PM)
 * @return boolean
 */
public boolean isFriendlyFireOn() 
{
	return friendlyFireOn;
}
/**
 * Insert the method's description here.
 * Creation date: (8/28/00 3:57:21 PM)
 * @param newC4Timer int
 */
public void setC4Timer(int newC4Timer) {
	c4Timer = newC4Timer;
}
/**
 * Insert the method's description here.
 * Creation date: (8/28/00 3:57:21 PM)
 * @param newCtWins int
 */
public void setCtWins(int newCtWins) {
	ctWins = newCtWins;
}
/**
 * Insert the method's description here.
 * Creation date: (9/3/00 7:16:09 AM)
 * @param currentName String
 * @param team int
 */
public void setCurrentTeam( String currentName, int team )
{
	GamePlayer p;
	if ( nameSource.containsKey( currentName ) )
	{
		p = (GamePlayer)nameSource.get( currentName );		

		// if this player doesn't already contain a record for this team
		Long w = new Long( p.getWonID() );
		if ( ! playerByTeam[team].containsKey( w ) )
		{
			GamePlayer gp = new GamePlayer();
			gp.setWonID( p.getWonID() );
			gp.setCurrentName( p.getCurrentName() );
			gp.setCurrentTeam( team );
			gp.setPlayer( p.getPlayer() );
			
			if ( p.getCurrentTeam() == GamePlayer.UNDEFINED )
			{
				gp.setReconnects( p.getReconnects() );
				// reconnects now belong to the terrorist
				p.setReconnects( 0 );
			}
				
			playerByTeam[team].put( w, gp );
		}
		p.setCurrentTeam( team );
	}
	else
		System.out.println( "Unable to assign to team--Unknown player: '" + currentName + "'" );	
}
/**
 * Insert the method's description here.
 * Creation date: (8/28/00 3:57:21 PM)
 * @param newDraws int
 */
public void setDraws(int newDraws) {
	draws = newDraws;
}
/**
 * Insert the method's description here.
 * Creation date: (8/13/00 11:47:19 AM)
 * @param newEndTime java.util.Date
 */
public void setEndTime(java.util.Date newEndTime) {
	endTime = newEndTime;
}
/**
 * Insert the method's description here.
 * Creation date: (8/13/00 12:03:00 PM)
 * @param newFriendlyFireOn boolean
 */
public void setFriendlyFireOn( boolean newFriendlyFireOn ) 
{
	friendlyFireOn = newFriendlyFireOn;
}
/**
 * Insert the method's description here.
 * Creation date: (8/28/00 2:46:22 PM)
 * @param newGamePlayers java.util.Hashtable
 */
void setGamePlayers(java.util.Hashtable newGamePlayers) {
	gamePlayers = newGamePlayers;
}
/**
 * Insert the method's description here.
 * Creation date: (9/9/00 7:32:04 AM)
 * @param newKickBans java.util.List
 */
public void setKickBans(java.util.List newKickBans) {
	kickBans = newKickBans;
}
/**
 * Insert the method's description here.
 * Creation date: (8/13/00 11:51:54 AM)
 * @param newMap java.lang.String
 */
public void setMapName( String newMap ) 
{
	if ( newMap != null )
	{
		int index = newMap.indexOf( '"' );		
		if ( index > 0 )
			newMap = newMap.substring( 0, index );
		mapName = newMap;
	}
	else
		mapName = "(Missing)";
		
}
/**
 * Set the list of players in a game.
 * used to re-create a game for reporting
 * Creation date: (9/1/00 8:21:51 AM)
 * @param players java.util.List a List of GamePlayer's
 */
public void setPlayers(List players) 
{
	if ( players == null )
		return;
		
	Iterator i = players.iterator();
	while ( i.hasNext() )
	{
		GamePlayer gp = (GamePlayer)i.next();
		addGamePlayer( gp );
		if ( gp.getCurrentName() != null )
			nameSource.put( gp.getCurrentName(), gp );
	}
}
/**
 * Insert the method's description here.
 * Creation date: (8/30/00 12:30:56 PM)
 * @param newServerId int
 */
public void setServerId(int newServerId) 
{
	serverId = newServerId;
}
/**
 * Insert the method's description here.
 * Creation date: (8/30/00 12:03:25 PM)
 * @param newServerRecord ServerRecord
 */
public void setServerRecord( ServerRecord newServerRecord ) 
{
	serverRecord = newServerRecord;
	serverId = serverRecord.getServerId();
}
/**
 * Insert the method's description here.
 * Creation date: (8/13/00 11:47:08 AM)
 * @param newStartTime java.util.Date
 */
public void setStartTime(java.util.Date newStartTime) {
	startTime = newStartTime;
}
/**
 * Insert the method's description here.
 * Creation date: (8/28/00 3:57:21 PM)
 * @param newTerrWins int
 */
public void setTerrWins(int newTerrWins) {
	terrWins = newTerrWins;
}
}
