package org.halfcat.halflife.info;

import org.halfcat.halflife.ServerRecord;
import org.halfcat.persistence.*;
import org.halfcat.halflife.cstrike.*;
import java.util.*;
/**
 * Insert the type's description here.
 * Creation date: (9/10/00 8:00:59 AM)
 * @author: Administrator
 */
public class RecentGames 
{
	// amount of time (in milliseconds) before the db will be re-queried
	long checkInterval = 60000;

	// # of games to query/populate/display
	int displaySize = 10;

	private static Hashtable serverRecords   = new Hashtable();
	private static Hashtable lastCheckedHash = new Hashtable();
	private static Hashtable gamesHash       = new Hashtable();
	
/**
 * RecentGames constructor comment.
 */
public RecentGames() 
{
	super();
}
/**
 * Insert the method's description here.
 * Creation date: (9/10/00 8:07:02 AM)
 * @return java.util.List
 */
private List fetchGames( Integer sid ) 
{
	return DataManager.fetchLimitedSorted( Game.class, "serverId", sid, "=", "startTime", false, displaySize );
}
/**
 * Return the checkInterval in seconds
 * Creation date: (9/10/00 8:41:21 AM)
 * @return long
 */
public long getCheckInterval() 
{
	return (checkInterval / 1000);
}
/**
 * Insert the method's description here.
 * Creation date: (9/10/00 8:41:21 AM)
 * @return int
 */
public int getDisplaySize() {
	return displaySize;
}
/**
 * Return the list of most recent games for this server. If it has been at least
 * fetchInterval seconds since the last update from the DB, then update from the
 * DB, then return;
 * Creation date: (9/10/00 8:13:27 AM)
 * @return java.util.List
 */
public List getGames( int sid ) 
{
	return getGames( new Integer( sid ) );
}
/**
 * Return the list of most recent games for this server. If it has been at least
 * fetchInterval seconds since the last update from the DB, then update from the
 * DB, then return;
 * Creation date: (9/10/00 8:13:27 AM)
 * @return java.util.List
 */
public List getGames( Integer sid ) 
{
	Date d = new Date();
	List games = null;
	if ( sid.intValue() > 0 )
	{
		Date lastChecked = (Date)lastCheckedHash.get( sid );
			
		if ( lastChecked == null || ( d.getTime() - checkInterval ) > lastChecked.getTime() )
		{
			lastCheckedHash.put( sid, d );
			games = fetchGames( sid );
			gamesHash.put( sid, games );
		}
		else
			games = (List)gamesHash.get( sid );
	}
	
	return games;
}
/**
 * Return the list of most recent games for this server. If it has been at least
 * fetchInterval seconds since the last update from the DB, then update from the
 * DB, then return;
 * Creation date: (9/10/00 8:13:27 AM)
 * @return java.util.List
 */
public List getGames( String address, int port ) 
{
	Integer sid = loadServerId( address, port	);
	return getGames( sid );
}
/**
 * Get the serverId for a ip/port pairing either from cache or from the DB
 * Creation date: (9/10/00 8:31:07 AM)
 * @return int
 */
private Integer loadServerId( String address, int port )
{
	String key = address + ":" + port;
	Integer serverId = null;
	if ( serverRecords.containsKey( key ) )
		return (Integer)serverRecords.get( key );
	else
	{
		try
		{
			ServerRecord sr;
			List l = DataManager.fetchLimited( ServerRecord.class, "address", address );
			Iterator i = l.iterator();
			Integer sid;
			while ( i.hasNext() )
			{
				sr = (ServerRecord)i.next();
				key = sr.getAddress() + ":" + sr.getPort();
				sid = new Integer( sr.getServerId() );
				serverRecords.put( key, sid );
					
				if ( sr.getPort() == port )
					serverId = sid;
			}
		}
		catch( Exception e )
		{
			e.printStackTrace();
		}
		return serverId;
	}
}
/**
 * Insert the method's description here.
 * Creation date: (9/10/00 8:41:21 AM)
 * @param newCheckInterval long
 */
void setCheckInterval(long newCheckInterval) {
	checkInterval = newCheckInterval;
}
/**
 * Insert the method's description here.
 * Creation date: (9/10/00 8:41:21 AM)
 * @param newDisplaySize int
 */
void setDisplaySize(int newDisplaySize) {
	displaySize = newDisplaySize;
}
}
