package org.halfcat.persistence;

import java.sql.*;
import java.util.*;

/**
 * The abstract superclass which all object maps will
 * probably want to extend.  Provides holders, getters, and setters
 * for various cached prepared statements
 * Creation date: (5/18/00 9:42:51 PM)
 * @author: Administrator
 */
public abstract class AbstractObjectMap
	implements ObjectMap
{
	/** Hashtable to cache prepared statements for insert,
	 * update, delete, and select + select by column.<BR>
	 * Key is {IUDS}[-colName] where the colName indicates
	 * a column referenced by a where clause
	 */
	private Hashtable m_statements = new Hashtable();


	static String brokerPackageExtension = "maps";
		
/**
 * AbstractBroker constructor comment.
 */
public AbstractObjectMap() 
{
	super();
}
/**
 * Remove all cached prepared statements for this ObjectMap
 * Creation date: (9/9/00 6:20:14 AM)
 */
protected synchronized void flushStatements() 
{
	m_statements.clear();
}
/**
 * Insert the method's description here.
 * Creation date: (7/30/00 8:24:24 AM)
 * @return java.lang.String
 */
public static String getBrokerPackageExtension() 
{
	return brokerPackageExtension;
}
/**
 * getFields method comment.
 */
public abstract String[] getFields();
/**
 * Insert the method's description here.
 * Creation date: (5/20/00 8:45:45 AM)
 * @param statementType java.lang.String
 * @exception java.sql.SQLException The exception description.
 */
public PreparedStatement getStatement( String statementType ) 
	throws SQLException
{
	if ( m_statements.containsKey( statementType ) )
		return (PreparedStatement)m_statements.get( statementType );

	PreparedStatement ps = null;
	
	return ps;	
}
/**
 * getTableName method comment.
 */
public abstract String getTableName();
/**
 * Insert the method's description here.
 * Creation date: (5/18/00 10:35:20 PM)
 * @param target java.lang.Object
 * @param rs java.sql.ResultSet
 * @exception java.sql.SQLException The exception description.
 */
public abstract Persistable populate(Persistable target, ResultSet rs) 
	throws 
		java.sql.SQLException;
/**
 * The SQL of this statement
 * will generally be the same for all objects (delete keyed on oid)
 * and thus not need to be overridden.
 * Creation date: (5/20/00 8:45:45 AM)
 * @param p org.halfcat.persistence.Persistable
 * @param ps java.sql.PreparedStatement
 * @exception java.sql.SQLException The exception description.
 */
public void prepareDelete(Persistable p, PreparedStatement ps) 
	throws SQLException 
{
	ps.setInt( 1, p.getObjectId() );
}
/**
 * Insert the method's description here.
 * Creation date: (5/20/00 8:45:45 AM)
 * @param p org.halfcat.persistence.Persistable
 * @param ps java.sql.PreparedStatement
 * @exception java.sql.SQLException The exception description.
 */
public abstract int prepareInsert(Persistable p, PreparedStatement ps) 
	throws SQLException ;
/**
 * Insert the method's description here.
 * Creation date: (5/20/00 8:45:45 AM)
 * @param p org.halfcat.persistence.Persistable
 * @param ps java.sql.PreparedStatement
 * @exception java.sql.SQLException The exception description.
 */
public void prepareSelectLimited( PreparedStatement ps, Object limiter ) 
	throws SQLException
{
	ps.setObject( 1, limiter );
}
/**
 * Insert the method's description here.
 * Creation date: (5/20/00 8:45:45 AM)
 * @param p org.halfcat.persistence.Persistable
 * @param ps java.sql.PreparedStatement
 * @exception java.sql.SQLException The exception description.
 */
public void prepareSelectOne( PreparedStatement ps, int oid ) 
	throws SQLException
{
	ps.setInt( 1, oid );
}
/**
 * prepare method comment.
 */
public int prepareUpdate(Persistable p, PreparedStatement ps) 
	throws SQLException
{
	int fieldCount = prepareInsert( p, ps );
	ps.setInt( ++fieldCount, p.getObjectId() );
	return fieldCount;
}
/**
 * Insert the method's description here.
 * Creation date: (5/20/00 9:37:36 AM)
 * @param statementType java.lang.String
 * @param ps java.sql.PreparedStatement
 */
public PreparedStatement putStatement( String statementType, PreparedStatement ps ) 
	throws SQLException
{
	m_statements.put( statementType, ps );
	return ps;
}
/**
 * Insert the method's description here.
 * Creation date: (7/30/00 8:24:24 AM)
 * @param newBrokerPackageExtension java.lang.String
 */
public static void setBrokerPackageExtension( String newBrokerPackageExtension ) 
{
	brokerPackageExtension = newBrokerPackageExtension;
}
}
