package org.halfcat.persistence;

import java.net.*;
import java.sql.*;
import java.util.*;

/**
 * This is the public API for talking to the database.  It has started down the
 * slippery slope of providing custom methods for various specialized queries,
 * and while I know that This Way Lies Madness, fixing it will have to wait for
 * a later version, as developing this library is a secondary priority, even within
 * the realm of my free time.
 * All of the various fetch...() methods are evil and must be destroyed.
 * Creation date: (5/19/00 10:07:14 PM)
 * @author: Administrator
 */
public class DataManager 
{
	private Hashtable m_objectMaps = new Hashtable();
	private static DataManager m_dataManager;

	// constants for keying of the PreparedStatement Hashtables
	public static String INSERT = "I";
	public static String UPDATE = "U";
	public static String DELETE = "D";
	public static String SELECT = "S";

	public static String OBJECT_ID = "oid";

	public static boolean debug = false;

/**
 * Insert the method's description here.
 * Creation date: (5/19/00 10:07:58 PM)
 */
private DataManager() {}
/**
 * Insert the method's description here.
 * Creation date: (5/23/00 10:05:20 PM)
 * @param p org.halfcat.persistence.Persistable
 * @exception java.sql.SQLException The exception description.
 */
public static void addBatch(Persistable p) 
	throws java.sql.SQLException 
{
	PreparedStatement ps = getStatement( p );
	ps.addBatch();
}
/**
 * Convenience method for deleting (saving an object flagged as DELETE)
 * Creation date: (5/20/00 4:03:08 PM)
 */
public static void delete( Persistable p )
	throws SQLException
{
	p.setObjectState( ObjectState.DELETE );
	save( p );
}
/**
 * Convenience method for deleting (saving an object flagged as DELETE)
 * Creation date: (5/20/00 4:03:08 PM)
 */
public static void deleteAll( Class p )
	throws SQLException
{	
	DataManager dm = getDataManager();
	try
	{
		DataSource.checkConnection();
		ObjectMap om = dm.getObjectMap( p.getName() );

		String sql = "DELETE FROM " + om.getTableName();
		Statement st = DataSource.getStatement();
		st.execute( sql );
	}
	catch( Exception e )
	{
		if ( debug )
			e.printStackTrace();
	}
	
}
/**
 * Convenience method for deleting (saving an object flagged as DELETE)
 * Creation date: (5/20/00 4:03:08 PM)
 */
public static void deleteLimited( Class p, String colName, Object limit )
	throws SQLException
{	
	DataManager dm = getDataManager();
	try
	{
		DataSource.checkConnection();

		ObjectMap om = dm.getObjectMap( p.getName() );
		PreparedStatement ps = dm.getDeleteStatementLimited( om, colName );
		om.prepareSelectLimited( ps, limit );
		ps.execute();
	}
	catch( Exception e )
	{
		if ( debug )
			e.printStackTrace();
	}
	
}
/**
 * Insert the method's description here.
 * Creation date: (5/23/00 10:05:53 PM)
 * @param persistable java.lang.Class
 * @exception java.sql.SQLException The exception description.
 */
public static void executeBatch(Class persistable) 
	throws java.sql.SQLException 
{
	PreparedStatement ps = null;
	try
	{
		ps = getStatement( (Persistable)persistable.newInstance() );
	}
	catch( IllegalAccessException iie )
	{
		iie.printStackTrace();
	}
	catch( InstantiationException iae )
	{
		iae.printStackTrace();
	}
	
	int[] ia = ps.executeBatch();
	System.out.println( "" + ia[0] );
}
/**
 * Fetch a specific Object from the database.  If more than
 * one object matches, only return the first one
 *
 * Creation date: (4/30/00 10:08:24 AM)
 * @return bing.Persistable
 * @param p bing.Persistable
 * @param keyColumn java.lang.String
 * @param keyValue  java.lang.Object
 */
public static List fetchAll( Class p )
{
	List retList = new ArrayList();
	DataManager dm = getDataManager();
	
	try
	{
		DataSource.checkConnection();
		ObjectMap om = dm.getObjectMap( p.getName() );
		PreparedStatement ps = dm.getSelectStatementAll( om );
		ResultSet rs = ps.executeQuery();
		
		while ( rs.next() )
			retList.add( om.populate( (Persistable)p.newInstance(), rs ) );
		rs.close();
	}
	catch( Exception e )
	{
		if ( debug )
			// if we've tried to reconnect and still failed
			e.printStackTrace();
	}	
	return retList;
}
/**
 * Very dodgy method to
 * execute a "SELECT WHERE field IN ( list )" statement
 * Creation date: (9/9/00 8:19:59 AM)
 * @return java.util.List
 * @param p java.lang.Class
 * @param keyColumn java.lang.String
 * @param inList java.util.List
 */
public static List fetchIn(Class p, String keyColumn, List inList) 
{
	ArrayList retList = new ArrayList();

	DataManager dm = getDataManager();
	ObjectMap om = dm.getObjectMap( p.getName() );

	StringBuffer sql = new StringBuffer( "SELECT oid," + dm.getFieldList( om ) + " FROM " );
	sql.append( om.getTableName() );
	sql.append( " WHERE " + keyColumn + " IN(" );
	
	// see if we can parse the first value as a number, and if we can, assume
	// this is a list of numbers--dodgy, but should work
	boolean addQuotes = ( ! ( inList.get( 0 ) instanceof Number ) );
	
	Iterator i = inList.iterator();
	while ( i.hasNext() )
	{
		if ( addQuotes )
			sql.append( '"' );

		sql.append( i.next() );

		if ( addQuotes )
			sql.append( '"' );

		if ( i.hasNext() )
			sql.append( "," );
	}
	sql.append( ")" );

	if ( debug )
		System.out.println( "sql: " + sql.toString() );
	try
	{
		Statement s = DataSource.getStatement();
		ResultSet rs = s.executeQuery( sql.toString() );
		
		while ( rs.next() )
			retList.add( om.populate( null, rs ) );
		rs.close();
	}
	catch( Exception e )
	{
		if ( debug )
			e.printStackTrace();
		else
			System.out.println( "Query failed: " + e.getMessage() );
	}
	
	return retList;
}
/**
 * Fetch a specific Object from the database.  If more than
 * one object matches, only return the first one
 *
 * Creation date: (4/30/00 10:08:24 AM)
 * @return bing.Persistable
 * @param p bing.Persistable
 * @param keyColumn java.lang.String
 * @param keyValue  java.lang.Object
 */
public static List fetchLimited( Class p, String keyColumn, Object keyValue )
{
	return fetchLimited( p, keyColumn, keyValue, "=" );
}
/**
 * Fetch a list of Objects from the database.  
 *
 * Creation date: (4/30/00 10:08:24 AM)
 * @return bing.Persistable
 * @param p bing.Persistable
 * @param keyColumn java.lang.String
 * @param keyValue  java.lang.Object
 */
public static List fetchLimited( Class p, String keyColumn, Object keyValue, 
	String comparator )
{	
	return fetchLimited( p, keyColumn, keyValue, false );
}
/**
 * Fetch a list of Objects from the database.  
 *
 * Creation date: (4/30/00 10:08:24 AM)
 * @return bing.Persistable
 * @param p bing.Persistable
 * @param keyColumn java.lang.String
 * @param keyValue  java.lang.Object
 */
public static List fetchLimited( Class p, String keyColumn, Object keyValue, 
	String comparator, boolean nocache )
{
	List retList = new ArrayList();
	DataManager dm = getDataManager();
	try
	{
		DataSource.checkConnection();
		ObjectMap om = dm.getObjectMap( p.getName() );
		PreparedStatement ps = dm.getSelectStatementLimited( om, keyColumn, comparator, nocache );
		om.prepareSelectLimited( ps, keyValue );
		ResultSet rs = ps.executeQuery();
		
		while ( rs.next() )
			retList.add( om.populate( (Persistable)p.newInstance(), rs ) );
		rs.close();
	}
	catch( Exception e )	
	{
		if ( debug )
			e.printStackTrace();
	}
	
	return retList;
}
/**
 * Fetch a specific Object from the database.  If more than
 * one object matches, only return the first one
 *
 * Creation date: (4/30/00 10:08:24 AM)
 * @return bing.Persistable
 * @param p bing.Persistable
 * @param keyColumn java.lang.String
 * @param keyValue  java.lang.Object
 */
public static List fetchLimited( Class p, String keyColumn, Object keyValue, boolean nocache )
{
	return fetchLimited( p, keyColumn, keyValue, "=", nocache );
}
/**
 * Fetch a list of Objects from the database.  This will be the last hacked, ugly
 * MacGuyver-esque code I put into DataManager.  I swear. 
 *
 * Creation date: (4/30/00 10:08:24 AM)
 * @return bing.Persistable
 * @param p bing.Persistable
 * @param keyColumn java.lang.String
 * @param keyValue  java.lang.Object
 */
public static List fetchLimitedSorted( Class p, String keyColumn, Object keyValue, 
	String comparator, String sortColumn, boolean ascending )
{
	return fetchLimitedSorted( p, keyColumn, keyValue, comparator, sortColumn, ascending, -1 );
}
/**
 * Fetch a list of Objects from the database.  This will be the last hacked, ugly
 * MacGuyver-esque code I put into DataManager.  I swear. 
 *
 * Creation date: (4/30/00 10:08:24 AM)
 * @return bing.Persistable
 * @param p bing.Persistable
 * @param keyColumn java.lang.String
 * @param keyValue  java.lang.Object
 */
public static List fetchLimitedSorted( Class p, String keyColumn, Object keyValue, 
	String comparator, String sortColumn, boolean ascending, int displaySize )
{
	List retList = new ArrayList();
	DataManager dm = getDataManager();
	try
	{
		DataSource.checkConnection();
		ObjectMap om = dm.getObjectMap( p.getName() );

		Statement s = DataSource.getStatement();
		
		StringBuffer sql = new StringBuffer( "SELECT oid, " + dm.getFieldList( om ) + " FROM " );
		sql.append( om.getTableName() );
		sql.append( " WHERE " );
		sql.append( keyColumn );
		sql.append( " " );
		sql.append( comparator );
		sql.append( " " );
		sql.append( keyValue );
		sql.append( " ORDER BY " );
		sql.append( sortColumn );
		if ( ascending )
			sql.append( " asc" );
		else
			sql.append( " desc" );
					
		ResultSet rs = s.executeQuery( sql.toString() );

		// Bad Developer!  Bad!  Bad!  No cookie for you!
		if ( displaySize > 0 )
		{
			int x = 0;
			while ( rs.next() && x++ < displaySize )
				retList.add( om.populate( (Persistable)p.newInstance(), rs ) );
		}
		else
			while ( rs.next() )
				retList.add( om.populate( (Persistable)p.newInstance(), rs ) );
		
		rs.close();
	}
	catch( Exception e )	
	{
		if ( debug )
			e.printStackTrace();
		else
			System.out.println( "Fetch error: " + e.getMessage() );
	}
		
	return retList;	
}
/**
 * Fetch a specific Object from the database based
 * on its objectId
 *
 * Creation date: (4/30/00 10:08:24 AM)
 * @return bing.Persistable
 * @param p java.lang.Class A reference to the class to query against
 * @param keyValue  java.lang.Object
 */
public static Persistable fetchOne( Class p, int objectId )
{
	Persistable retP = null;
	DataManager dm = getDataManager();
	try
	{
		DataSource.checkConnection();
		ObjectMap om = dm.getObjectMap( p.getName() );
		PreparedStatement ps = dm.getSelectStatementOne( om );
		om.prepareSelectOne( ps, objectId );
		try
		{
			ResultSet rs = ps.executeQuery();

			if (rs.next())
				retP = om.populate( (Persistable)p.newInstance(), rs );

			rs.close();
		}
		catch( NullPointerException npe )
		{
			System.out.println( "Null Pointer during fetchOne(): " + npe.getMessage() );
		}
		
		// leave the statement open for re-use
	}
	catch (Exception e)
	{
		if ( debug )
			System.out.println( "DataManager.fetchOne(): " + e.getMessage() );
	}
	return retP;
}
/**
 * Insert the method's description here.
 * Creation date: (9/9/00 6:23:12 AM)
 */
protected void flushCache() 
{
	Iterator i = m_objectMaps.values().iterator();
	while ( i.hasNext() )
	{
		((AbstractObjectMap)i.next()).flushStatements();
	}
}
/**
 * Insert the method's description here.
 * Creation date: (5/19/00 10:08:30 PM)
 * @return org.halfcat.persistence.DataManager
 */
protected static DataManager getDataManager() 
{
	if ( m_dataManager == null )
		m_dataManager = new DataManager();

	return m_dataManager;
}
/**
 * Insert the method's description here.
 * Creation date: (5/18/00 10:23:56 PM)
 * @return java.sql.PreparedStatement
 */
private PreparedStatement getDeleteStatement( ObjectMap om )
	throws SQLException
{
	PreparedStatement ps = om.getStatement( DELETE );
	if ( ps == null )	
	{
		StringBuffer sql = new StringBuffer( "DELETE FROM " );
		sql.append( om.getTableName() );
		sql.append( " WHERE oid = ?");			

		ps = DataSource.getPreparedStatement( sql.toString() );
		om.putStatement( DELETE, ps );
	}
	return ps;	
}
/**
 * Delete rows based on the value of the specified column
 * Creation date: (5/20/00 10:12:24 AM)
 * @return java.sql.PreparedStatement
 * @param om org.halfcat.persistence.ObjectMap
 * @exception java.sql.SQLException The exception description.
 */
private PreparedStatement getDeleteStatementLimited( ObjectMap om, String colName ) 
	throws java.sql.SQLException 
{
	PreparedStatement ps;
	ps = om.getStatement( DELETE + "-" + colName );
	
	if ( ps == null )
	{
		StringBuffer sql = new StringBuffer( "DELETE FROM " );
		sql.append( om.getTableName() );
		sql.append( " WHERE " + colName + "=? " );
		ps = DataSource.getPreparedStatement( sql.toString() );
		om.putStatement( DELETE + "-" + colName, ps );
	}
	return ps;	
}
/**
 * Insert the method's description here.
 * Creation date: (9/1/00 3:29:19 PM)
 * @return java.lang.String
 * @param om org.halfcat.persistence.ObjectMap
 */
private String getFieldList(ObjectMap om) 
{
	StringBuffer sb = new StringBuffer();
	String[] fields = om.getFields();
	for ( int x = 0; x < fields.length; x++ )
	{
		sb.append( fields[x] );
		if ( (x + 1) < fields.length )
			sb.append( "," );				
	}
	return sb.toString();
}
/**
 * Insert the method's description here.
 * Creation date: (5/18/00 10:23:56 PM)
 * @return java.sql.PreparedStatement
 */
private PreparedStatement getInsertStatement( ObjectMap om )
	throws SQLException
{
	PreparedStatement ps;
	ps = om.getStatement( INSERT );
	
	if ( ps == null )
	{
		StringBuffer sql = new StringBuffer( "INSERT INTO " );
		sql.append( om.getTableName() );
		sql.append( " ( " );
		String[] fields = om.getFields();
		int fieldsLen = fields.length;

		for ( int x = 0; x < fieldsLen; x++ )
		{
			sql.append( fields[x] );
			if ( x < (fieldsLen - 1) )
				sql.append( "," );
		}
		sql.append( " ) VALUES ( " );
		for ( int x = 0; x < fieldsLen; x++ )
		{
			sql.append( "?" );
			if ( x < (fieldsLen - 1) )
				sql.append( "," );
		}
		sql.append( " )" );
		ps = DataSource.getPreparedStatement( sql.toString() );
		om.putStatement( INSERT, ps );
	}
	return ps;	
}
/**
 * Insert the method's description here.
 * Creation date: (5/20/00 11:01:55 AM)
 */
private ObjectMap getObjectMap( String mapName )
{
	ObjectMap om = null;
	try
	{
		if (m_objectMaps.containsKey(mapName))
			return (ObjectMap) m_objectMaps.get(mapName);
		else
		{
			int classIndex = mapName.lastIndexOf( "." ) + 1;
			StringBuffer sb = new StringBuffer( mapName.substring( 0, classIndex ) );
			// TO-DO:  Fix this hardcoding
			sb.append( "maps" );
			sb.append( "." );
			sb.append( mapName.substring( classIndex ) );
			sb.append( "Map" );
			om = (ObjectMap) Class.forName( sb.toString() ).newInstance();
			m_objectMaps.put( mapName, om );
		}
	}
	catch (Exception e)
	{
		// Temporary until I do exceptions
		System.out.println( "DataManager.getObjectMap() failed: " + e.getMessage() );
	}
	return om;
}
/**
 * Insert the method's description here.
 * Creation date: (5/20/00 10:12:24 AM)
 * @return java.sql.PreparedStatement
 * @param om org.halfcat.persistence.ObjectMap
 * @exception java.sql.SQLException The exception description.
 */
private PreparedStatement getSelectStatementAll( ObjectMap om ) 
	throws java.sql.SQLException 
{
	PreparedStatement ps;
	ps = om.getStatement( SELECT  );
	
	if ( ps == null )
	{
		
		String sql = "SELECT oid, " + getFieldList( om ) + " FROM " + om.getTableName();
		ps = DataSource.getPreparedStatement( sql );
		om.putStatement( SELECT, ps );
	}
	return ps;	
}
/**
 * Insert the method's description here.
 * Creation date: (5/20/00 10:12:24 AM)
 * @return java.sql.PreparedStatement
 * @param om org.halfcat.persistence.ObjectMap
 * @exception java.sql.SQLException The exception description.
 */
private PreparedStatement getSelectStatementLimited( ObjectMap om, String colName ) 
	throws java.sql.SQLException 
{
	return getSelectStatementLimited( om, colName, "=", false );
}
/**
 * Insert the method's description here.
 * Creation date: (5/20/00 10:12:24 AM)
 * @return java.sql.PreparedStatement
 * @param om org.halfcat.persistence.ObjectMap
 * @exception java.sql.SQLException The exception description.
 */
private PreparedStatement getSelectStatementLimited( ObjectMap om, String colName, 
		String comparator ) 
	throws java.sql.SQLException 
{
	return getSelectStatementLimited( om, colName, comparator, false );
}
/**
 * Insert the method's description here.
 * Creation date: (5/20/00 10:12:24 AM)
 * @return java.sql.PreparedStatement
 * @param om org.halfcat.persistence.ObjectMap
 * @exception java.sql.SQLException The exception description.
 */
private PreparedStatement getSelectStatementLimited( ObjectMap om, String colName, String comparator, boolean nocache ) 
	throws java.sql.SQLException 
{
	PreparedStatement ps = null;
	if ( ! nocache )
		ps = om.getStatement( SELECT + "-" + comparator + "-" + colName );
	
	if ( ps == null )
	{
		StringBuffer sql = new StringBuffer( "SELECT oid, " + getFieldList( om ) + " FROM " );
		sql.append( om.getTableName() );
		sql.append( " WHERE " + colName + " " + comparator + " ? " );
		ps = DataSource.getPreparedStatement( sql.toString() );
		if ( ! nocache )	
			om.putStatement( SELECT + "-" + colName, ps );
	}
	return ps;	
}
/**
 * Insert the method's description here.
 * Creation date: (5/20/00 10:12:24 AM)
 * @return java.sql.PreparedStatement
 * @param om org.halfcat.persistence.ObjectMap
 * @exception java.sql.SQLException The exception description.
 */
private PreparedStatement getSelectStatementOne( ObjectMap om ) 
	throws java.sql.SQLException 
{
	return getSelectStatementLimited( om, "oid" );
}
/**
 * Return a populated @PreparedStatement or null if the
 * statement could not be populated properly
 * Creation date: (5/23/00 9:59:22 PM)
 * @return java.sql.PreparedStatement
 * @param p org.halfcat.persistence.Persistable
 * @exception java.sql.SQLException The exception description.
 */
private static PreparedStatement getStatement(Persistable p) 
	throws java.sql.SQLException 
{
	if ( p == null )
		return null;
	
	DataManager dm = getDataManager();	
	PreparedStatement ps = null;
	ObjectMap om = dm.getObjectMap( p.getClass().getName() );
	int state = p.getObjectState();
	int newState = -1;

	if ( state == ObjectState.CLEAN )
		state = ObjectState.DIRTY;
			
	if ( state == ObjectState.NEW )
	{
		ps = dm.getInsertStatement( om );
		om.prepareInsert( p, ps );
		newState = ObjectState.CLEAN;
	}
	else if ( state == ObjectState.DIRTY )
	{
		ps = dm.getUpdateStatement( om );
		om.prepareUpdate( p, ps );
		newState = ObjectState.CLEAN;
	}
	else if ( state == ObjectState.DELETE )
	{
		ps = dm.getDeleteStatement( om );
		om.prepareDelete( p, ps );
		newState = ObjectState.DELETED;
	}
	// if the object is clean, then don't save it, but warn in the console
	else
	{
		System.out.println( "Attempt to save object with invalid state: " + 
				om.getTableName() + " " + p.getObjectId() );
		return null;
	}
	
	p.setObjectState( newState );
	return ps;
}
/**
 * Insert the method's description here.
 * Creation date: (5/18/00 10:23:56 PM)
 * @return java.sql.PreparedStatement
 */
private PreparedStatement getUpdateStatement( ObjectMap om )
	throws SQLException
{
	PreparedStatement ps;
	ps = om.getStatement( UPDATE );
	
	if ( ps == null )
	{
		StringBuffer sql = new StringBuffer( "UPDATE " );
		sql.append( om.getTableName() );
		sql.append( " SET " );
		String[] fields = om.getFields();
		int fieldsLen = fields.length;
		for ( int x = 0; x < fieldsLen; x++ )
		{
			sql.append( fields[x] );
			sql.append( "=?" );
			if ( x < (fieldsLen - 1) )
				sql.append( "," );
		}
		sql.append( " WHERE oid = ?");			

		ps = DataSource.getPreparedStatement( sql.toString() );			
		om.putStatement( UPDATE, ps );
	}
	return ps;	
}
/**
 * Save an object to the database
 * Creation date: (5/19/00 10:10:20 PM)
 * @param p org.halfcat.persistence.Persistable
 */
public static void save( Persistable p )
	throws SQLException
{
	if ( p == null )
		return;
		
	DataSource.checkConnection();
		
	int state = p.getObjectState();
	PreparedStatement ps = getStatement( p );
	int newState = p.getObjectState();

	// we want to wind up with a fully processed PreparedStatement that we
	// can then execute and manage the results from here
	boolean failed = ps.execute();

	if ( failed )
		p.setObjectState( state );
	else 
	{
		// if we managed to write a row to the db, then we can get the status out of the
		// Postgres-specific result set, then parse that to get the new object's ID
		if ( state != ObjectState.DIRTY && newState != ObjectState.DELETED )
		{
			org.postgresql.ResultSet rss = (org.postgresql.ResultSet) ps.getResultSet();
//			ResultSet rss = ps.getResultSet();
//			int oidColumn = rss.findColumn( "oid" );
			try
			{
				String status = rss.getStatusString();
				String substring = status.substring(status.indexOf(' ') + 1, status.lastIndexOf(' '));
				int subint = Integer.parseInt(substring);
			
				p.setObjectId(subint);						
			}
			catch( Exception e )
			{
				System.out.println( "Problem processing resultSet after save: " + e.getMessage() );
			}
			rss.close();
		}
		p.setObjectState( newState );
	}	
}
}
