package org.halfcat.persistence;

import java.io.*;
import java.util.*;

//Title:        Your Product Name
//Version:      
//Copyright:    Copyright (c) 1999
//Author:       Chandler Howell
//Company:      WDR
//Description:  Your description
import java.sql.*;

public class DataSource
{
	private static Connection m_dbConn;
	private static boolean m_debug = true;

	private static String dbUrl = null;
	private static String dbLogin = null;
	private static String dbPassword = null;
  private DataSource()
  {
  }    
/**
 * Check the connection to the database, and if it has failed,
 * clear any cached Statements and reconnect
 * Creation date: (9/8/00 5:20:52 PM)
 * @exception java.sql.SQLException The exception description.
 */
public static void checkConnection() 
	throws java.sql.SQLException 
{
	if ( m_dbConn == null || m_dbConn.isClosed() )
	{
		System.out.println( "Reconnecting now" );
		DataManager.getDataManager().flushCache();
		reconnect();
	}
}
/**
 * Close the database connection
 * Creation date: (4/30/00 8:48:42 AM)
 */
public static void closeConnection()
{
	try
	{
		m_dbConn.close();
	}
	catch( SQLException se )
	{
		se.printStackTrace();
	}
}
/**
 * Connect to the Database using the parameters specified in
 * file @propertiesFilename.<BR>
 * Recognized properties are:<BR>
 * <OL>
 * <LI><B>dbHost</B> -- The hostname or IP Address of the database<BR>
 * <LI><B>dbName</B> -- The name of the database on the server
 * <LI><B>dbLogin</B> -- The userId for this database
 * <LI><B>dbPasswd</B> -- The password to use with this login
 * <LI><B>dbDriver</B> -- (Optional)If the dbDriver is not specified
 * in the properties file, then it will be looked for as a separate 
 * property, "jdbc.drivers", taken from the system properties
 * </OL>
 * Creation date: (5/23/00 8:47:48 PM)
 * @param propertiesName java.lang.String
 * @exception java.sql.SQLException If an error occurs connecting
 * to the database
 * @exception java.io.IOException If the system is unable to locate
 * or read the specified @propertiesFilename
 */
public static void connectToDB( String propertiesFilename ) 
	throws SQLException,
		IOException
{
	Properties dbProps = new Properties();

	FileInputStream fis = new FileInputStream( propertiesFilename );
	dbProps.load( fis );

	String dbDriver = dbProps.getProperty( "dbDriver", ""  );
	String dbHost   = dbProps.getProperty( "dbHost",   ""    );
	String dbName   = dbProps.getProperty( "dbName",   "" );
	String dbLogin  = dbProps.getProperty( "dbLogin",  ""  );
	String dbPasswd = dbProps.getProperty( "dbPasswd", ""  );

	if ( dbDriver == null || dbDriver.equals( "" ) )
		dbDriver = System.getProperty( "jdbc.drivers" );
	dbDriver = dbDriver.substring( 0, dbDriver.indexOf( "." ) );

	StringBuffer dbURL = new StringBuffer( "jdbc:" );
	dbURL.append( dbDriver );
	dbURL.append( "://" );
	dbURL.append( dbHost );
	dbURL.append( "/" );
	dbURL.append( dbName );
	
	DataSource.connectToDB( dbURL.toString(), dbLogin, dbPasswd );
}
public static void connectToDB(String url, String newDbLogin, String newDbPasswd)
{
	// Load the driver
	// this is now done on the command-line with:
	//   $ java -Djdbc.drivers=postgresql.Driver package.class

	dbUrl = url;
	dbLogin = newDbLogin;
	dbPassword = newDbPasswd;
	
	// Connect to database
	if (m_dbConn == null)
	{
		if (m_debug)
			System.out.println("Connecting to Database URL = " + url);
		try
		{
			m_dbConn = DriverManager.getConnection(url, dbLogin, dbPassword);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
}
/**
 * Insert the method's description here.
 * Creation date: (5/25/00 9:50:47 PM)
 */
public static String createDbUrl( String dbHost, String dbName, int dbPort, String dbDriver )
{
	try 
	{
		Class.forName("org.postgresql.Driver");
	}
	catch(Exception e) 
	{
		e.printStackTrace();
	}


	// if we didn't get it passed, try and calc it
	if ( dbDriver == null || dbDriver.equals( "" ) )
		dbDriver = System.getProperty( "jdbc.drivers" );

	dbDriver = "postgresql";
//	int periodIndex = dbDriver.indexOf( "." );

//	if ( periodIndex > 0 )
//		dbDriver = dbDriver.substring( 0, periodIndex );

	StringBuffer dbURL = new StringBuffer( "jdbc:" );
	dbURL.append( dbDriver );
	dbURL.append( ":" );
	dbURL.append( "//" );
	dbURL.append( dbHost );
	dbURL.append( ":" );
	dbURL.append( "" + dbPort );
	dbURL.append( "/" );
	dbURL.append( dbName );

	return dbURL.toString();	
}
/**
 * Insert the method's description here.
 * Creation date: (5/25/00 9:50:47 PM)
 */
public static String createDbUrl( String dbHost, String dbName, String dbDriver )
{
	try 
	{
		Class.forName("org.postgresql.Driver");
	}
	catch(Exception e) 
	{
		e.printStackTrace();
	}


	// if we didn't get it passed, try and calc it
	if ( dbDriver == null || dbDriver.equals( "" ) )
		dbDriver = System.getProperty( "jdbc.drivers" );

	dbDriver = "postgresql";
//	int periodIndex = dbDriver.indexOf( "." );

//	if ( periodIndex > 0 )
//		dbDriver = dbDriver.substring( 0, periodIndex );

	StringBuffer dbURL = new StringBuffer( "jdbc:" );
	dbURL.append( dbDriver );
	dbURL.append( "://" );
	dbURL.append( dbHost );
	dbURL.append( "/" );
	dbURL.append( dbName );

	return dbURL.toString();	
}
/**
 * This method may disappear, but is used right now to expose
 * the underlying API's on the connection for the DatabaseBrowser
 * Creation date: (5/21/00 9:20:55 AM)
 * @return java.sql.Connection
 */
public static Connection getConnection()
	throws SQLException
{
	if ( m_dbConn == null || m_dbConn.isClosed() )
		reconnect();

	return m_dbConn;
}
public static PreparedStatement getPreparedStatement( String sql ) 
	throws 
		java.sql.SQLException
{
	if ( m_dbConn == null || m_dbConn.isClosed() )
		reconnect();
		
	return m_dbConn.prepareStatement( sql );
}
public static Statement getStatement() 
	throws 
		java.sql.SQLException
{
	if ( m_dbConn == null || m_dbConn.isClosed() )
		reconnect();
		
	return m_dbConn.createStatement();
}
/**
 * Insert the method's description here.
 * Creation date: (4/30/00 10:01:36 AM)
 * @return boolean
 */
public static boolean isConnected()
	throws SQLException
{
	return ( m_dbConn != null && ( ! m_dbConn.isClosed() ) );
}
/**
 * Insert the method's description here.
 * Creation date: (9/7/00 7:07:10 PM)
 */
public static void reconnect() 
{
	System.out.println( "*****\n***** RECONNECTING TO DB\n*****" );
//	connectToDB( dbUrl, dbLogin, dbPassword );
		try
		{
			m_dbConn = DriverManager.getConnection( dbUrl, dbLogin, dbPassword );
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

}
}
